///////////////////////////////////////////////////////////////////////////////
// Class DVDx                                                                //
// Created by Hell Hibou (2009)                                              //
// Part of MyFS library                                                      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


extern "C"
{ 
	#include <di/di.h>
}

#include <string.h>
#include <gccore.h>

#include "dvdx.hpp"
#include "FS.hpp"

///////////////////////////////////////////////////////////////////////////////

C_DVDx::T_Flags	C_DVDx::Flags	= { 0, 0 };
u32 C_DVDx::BufferBlocId		= 0;
u8  C_DVDx::Buffer [2048] ATTRIBUTE_ALIGN (32);

///////////////////////////////////////////////////////////////////////////////

int C_DVDx::Init ()
{
	int RetVal;
	
	Flags.BufferStatus = 0;
	BufferBlocId = 0;
	memset (Buffer, 0x00, sizeof (C_DVDx::Buffer));
	RetVal = DI_Init();
	
	if (RetVal >= 0) 
	{
		Flags.IsInit = 1; 
		return NO_ERROR;
	}  
	
	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////

int C_DVDx::Deinit () 
{
	Flags.IsInit = 0;
	BufferBlocId = 0;
	DI_Close(); 
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_DVDx::Mount (void) 
{
	u32 RetVal;
	
	if (Flags.IsInit == 0) { return ERROR_NOT_INITIALIZED; }
	DI_GetCoverRegister(&RetVal);
	if(RetVal & 0x1) { return FS_ERROR_DEVICE_NOT_FOUND; }
	DI_Mount();
	while (DI_GetStatus() & DVD_INIT);	
	Flags.BufferStatus = 0;
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_DVDx::Unmount() { return NO_ERROR; }

///////////////////////////////////////////////////////////////////////////////

int C_DVDx::ReadBloc (void * BufferOut, u32 BlocId)
{
	int RetVal;

	if (Flags.IsInit == 0) { return ERROR_NOT_INITIALIZED; }
	if ((BufferBlocId != BlocId) || (Flags.BufferStatus == 0))
	{
		DCInvalidateRange ((void *) &Buffer, 2048);
		
		RetVal = DI_ReadDVD (&Buffer, 1, BlocId);
		if (RetVal) { return RetVal; }
		
		Flags.BufferStatus = 1;
		BufferBlocId = BlocId;
	}

	memcpy (BufferOut, Buffer, 2048);
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////
