///////////////////////////////////////////////////////////////////////////////
// Classes C_File and C_FileSystem                                           //
// Created by Hell Hibou (2009)                                              //
// Part of MyFS library                                                      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#ifndef _CLASS_UOBIHLLEH_C_FS
#define _CLASS_UOBIHLLEH_C_FS

#include <gccore.h>
#include <stdio.h>

#include "T_error.h"
#include "strconv.h"

#define DIR_ATTRIB_DIRECTORY	(0x01)

#define FILE_OPEN_READ			 1
#define FILE_OPEN_WRITE			 2
#define FILE_OPEN_CREATE		 4
#define FILE_OPEN_TRUNCATE		 8
#define FILE_OPEN_APPEND		16

#define FILE_OPEN_RW			(FILE_OPEN_READ | FILE_OPEN_WRITE)


#define FILE_SEEK_SET	1
#define FILE_SEEK_CUR	2
#define FILE_SEEK_END	3


#define FS_EOF					 100
#define	FS_WARNING_FILES_OPENED	 101


#define FS_ERROR_IO        			-(EIO)
#define FS_ERROR_DEVICE_NOT_FOUND	-(__ELASTERROR + 101)
#define	FS_ERROR_NOT_OPENED			-(__ELASTERROR + 102)
#define	FS_ERROR_FILE_OPENED		-(__ELASTERROR + 103)
#define FS_ERROR_NOT_FOUND			-(__ELASTERROR + 104)
#define FS_ERROR_NOT_EXECUTED		-(__ELASTERROR + 105)
#define FS_ERROR_CANT_LOAD_FILE		-(__ELASTERROR + 106)
#define FS_ERROR_EXIST				-(__ELASTERROR + 107)
#define FS_ERROR_NOT_EXIST			-(__ELASTERROR + 108)
#define FS_ERROR_READ_ONLY			-(__ELASTERROR + 109)
#define FS_ERROR_OUT_OF_SPACE		-(__ELASTERROR + 110)
#define FS_ERROR_NOT_DIRECTORY		-(__ELASTERROR + 111)
#define FS_ERROR_NOT_FILE			-(__ELASTERROR + 112)
#define FS_ERROR_NOT_SUPPORTED		-(__ELASTERROR + 113)
#define FS_ERROR_NOT_MOUNTED		-(__ELASTERROR + 114)

///////////////////////////////////////////////////////////////////////////////
// Class C_File                                                              //
///////////////////////////////////////////////////////////////////////////////

class C_File
{
	public:
		const u32 & Size;
		const u32 & Pos;
		
		C_File ();
		
		virtual ~C_File ();
		virtual int open  (const char *, u16 Mode)			= 0;
		virtual int close (void) 							= 0;
		virtual int seek  (u8 Mode, u32 Pos) 				= 0;
		virtual s32 read  (void * Buffer, s32 Size)			= 0;
		virtual s32 write (const void * Buffer, s32 Size)	= 0;
		virtual int eof	  (void)							= 0;

	protected:
		u32	FileSize;
		u32 FilePos;
};	


///////////////////////////////////////////////////////////////////////////////
// Class C_Dir                                                               //
///////////////////////////////////////////////////////////////////////////////

class C_Dir
{
	public:
		const char *	Name;
		u16				Attrib;
		unsigned long	Size;
		
		C_Dir();
		virtual ~C_Dir    (void);
		virtual int open  (const char *)	= 0;
		virtual int close ()				= 0;
		virtual int	next  ()				= 0;
		virtual int reset ()				= 0;
		virtual int mkdir (const char *)	= 0;

	protected:
		char * 	V_Name;
};



///////////////////////////////////////////////////////////////////////////////
// Class C_FileSystem                                                        //
///////////////////////////////////////////////////////////////////////////////

class C_FileSystem
{
	public:	
		void * 	LoadFile  (const char *, u32 *FileSize = NULL);
		int		FileExist (const char *);
		
		virtual ~C_FileSystem		(void);
		virtual int Mount			(const char * = NULL);
		virtual int Unmount			(void);
		virtual C_File * NewFile	(void)				= 0;
		virtual C_Dir  * NewDir		(void)				= 0;
		virtual int mkdir 			(const char *)		= 0;
		
		virtual const char * GetStringParam		(void)	= 0;
		virtual const char * GetFSname			(void)	= 0;
		virtual T_String_PageCode  GetPageCode	(void)	= 0;
};

#endif

///////////////////////////////////////////////////////////////////////////////
