///////////////////////////////////////////////////////////////////////////////
// Classes C_FS_ISO9660, C_FS_ISO9660::C_File and C_FS_ISO9660::C_Dir        //
// Created by Hell Hibou (2009)                                              //
// Part of MyFS library                                                      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <string.h>
#include <malloc.h>

#include "FS_ISO9660.hpp"
#include "dvdx.hpp"


///////////////////////////////////////////////////////////////////////////////
// Class C_FS_ISO9660_Internal                                               //
///////////////////////////////////////////////////////////////////////////////

#define ENTRY_BUFFER_SIZE	16

#define ENCODAGE_ISO9660	1
#define ENCODAGE_Joliet		2

u32  C_FS_ISO9660_Internal::RootBlocId		 = 0;
u32  C_FS_ISO9660_Internal::RootBlocSize	 = 0;
u8   C_FS_ISO9660_Internal::Encodage		 = 0;
u32	 C_FS_ISO9660_Internal::ObjectCount		 = 0;
u16  C_FS_ISO9660_Internal::EntryBufferIndex = 0;
C_FS_ISO9660_Internal::T_EntryBuffer * C_FS_ISO9660_Internal::EntryBuffer = NULL;

///////////////////////////////////////////////////////////////////////////////

C_FS_ISO9660_Internal::C_FS_ISO9660_Internal()
{
#if (ENTRY_BUFFER_SIZE > 0)
	if (EntryBuffer == NULL)
	{
		EntryBuffer = (C_FS_ISO9660_Internal::T_EntryBuffer *) malloc (sizeof(C_FS_ISO9660_Internal::T_EntryBuffer) * ENTRY_BUFFER_SIZE);
		EntryBufferIndex = 0;
		
		if (EntryBuffer != NULL)
		{
			for (int Boucle = 0; Boucle < ENTRY_BUFFER_SIZE; Boucle++)
			{
				EntryBuffer[Boucle].EntryName	= NULL;
				EntryBuffer[Boucle].BlocId		= 0;
				EntryBuffer[Boucle].Fileflag	= 0;
				EntryBuffer[Boucle].DataLength	= 0;	
			}
		}
	}
	
	ObjectCount++;
#endif
}

///////////////////////////////////////////////////////////////////////////////

C_FS_ISO9660_Internal::~C_FS_ISO9660_Internal()
{
#if (ENTRY_BUFFER_SIZE > 0)	
	ObjectCount--;
	if ((ObjectCount == 0) && (EntryBuffer != NULL))
	{
		free(EntryBuffer);
		EntryBuffer = NULL;	
	}
#endif
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660_Internal::ReadHeader ()
{
	u32 BlocId = ISO9660_VD_LBA_BEGIN - 1;
	u32 VDblocId	= 0;
	RootBlocId		= 0;
	RootBlocSize	= 0;
	Encodage		= 0;
	u8 * DVDbuffer;
	u8 Loop;

	DVDbuffer = (u8*)malloc (ISO9660_BLOC_SIZE);
	if (DVDbuffer == NULL) { return ERROR_OUT_OF_MEMORY; }
	Encodage = 0;
	Loop = 1;

	do
	{
		BlocId ++;
		
		if (C_DVDx::ReadBloc (DVDbuffer, BlocId) < 0) 
		{ 
			free(DVDbuffer); 
			return FS_ERROR_IO; 
		}
		
		if (strncmp((char*) &((T_ISO9660_VD*)DVDbuffer)->Magic, ISO9660_VD_MAGIC, ISO9660_VD_MAGIC_SIZE) == 0)
		{		
			if (((T_ISO9660_VD*)DVDbuffer)->Type == ISO9660_VD_PVD) 
			{ 
				VDblocId = BlocId;
				Encodage = ENCODAGE_ISO9660;
			}
			
			else if ( ((T_ISO9660_VD*)DVDbuffer)->Type == ISO9660_VD_SVD)
			{
				// Joliet ?
				if ( (strcmp( ((T_ISO9660_VD*)DVDbuffer)->VD.SVD.EscapeSequences, ISO9660_ESCAPESEQUENCE_JOLIET_LEVEL1) == 0)
				   ||(strcmp( ((T_ISO9660_VD*)DVDbuffer)->VD.SVD.EscapeSequences, ISO9660_ESCAPESEQUENCE_JOLIET_LEVEL2) == 0)
				   ||(strcmp( ((T_ISO9660_VD*)DVDbuffer)->VD.SVD.EscapeSequences, ISO9660_ESCAPESEQUENCE_JOLIET_LEVEL3) == 0) )
				{
					RootBlocId		= ((T_ISO9660_VD*)DVDbuffer)->VD.SVD.RootDir.ExtentLoc.Value;
					RootBlocSize	= ((T_ISO9660_VD*)DVDbuffer)->VD.SVD.RootDir.DataLength.Value;
					Encodage		= ENCODAGE_Joliet;
					free (DVDbuffer); 
					return NO_ERROR;
				}
			}
			
			else if (((T_ISO9660_VD*)DVDbuffer)->Type == ISO9660_VD_VDST) break;
		} else break;
	} while (BlocId <= ISO9660_VD_LBA_END);
	
	if ((BlocId != VDblocId) && (VDblocId != 0))
	{
		if (C_DVDx::ReadBloc (DVDbuffer, VDblocId) < 0) 
		{
			free(DVDbuffer); 
			return FS_ERROR_IO; 
		}
	}
	
	if (VDblocId != 0)
	{
		Encodage		= ENCODAGE_ISO9660;
		RootBlocId		= ((T_ISO9660_VD*)DVDbuffer)->VD.PVD.RootDir.ExtentLoc.Value;
		RootBlocSize	= ((T_ISO9660_VD*)DVDbuffer)->VD.PVD.RootDir.DataLength.Value;
		free(DVDbuffer); 
		return NO_ERROR;
	}

	free(DVDbuffer); 
	return FS_ERROR_DEVICE_NOT_FOUND;
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660_Internal::OpenDir (u32 BlocId, u32 Size, ISO9660_DirPtr * DirPtr)
{
	DirPtr->Info		= NULL;
	DirPtr->PosInBloc   = 0;
	DirPtr->CurrentBloc = BlocId;
	DirPtr->LastBloc    = BlocId + (Size / ISO9660_BLOC_SIZE);
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660_Internal::ReadDir (ISO9660_DirPtr * DirPtr)
{
	u8 * Dir;
	u8 * DVDbuffer;
	
	if (DirPtr == NULL)		{ return ERROR_NULL_VALUE; }
	
	DVDbuffer = (u8*) malloc (ISO9660_BLOC_SIZE);
	if (DVDbuffer == NULL)	{ return ERROR_OUT_OF_MEMORY; }
	
	if (DirPtr->Info != NULL)	{ Dir = (u8*) DVDbuffer + DirPtr->PosInBloc; }
	else 						{ Dir = (u8*) DVDbuffer; }

	if (C_DVDx::ReadBloc (DVDbuffer, DirPtr->CurrentBloc) < 0) 
	{
		free (DVDbuffer);
		return FS_ERROR_IO; 
	}

	do
	{
		if (DirPtr->Info != NULL)
		{
			if ( (DirPtr->PosInBloc + ((T_ISO9660_DR *)Dir)->DRsize <= ISO9660_BLOC_SIZE) 
			&& (((T_ISO9660_DR *)Dir)->DRsize != 0) ) 
			{
				Dir +=  ((T_ISO9660_DR *)Dir)->DRsize;
				DirPtr->PosInBloc = Dir - DVDbuffer;
			}
			else
			{
				DirPtr->CurrentBloc++;
				
				if (DirPtr->CurrentBloc < DirPtr->LastBloc)
				{
					if (C_DVDx::ReadBloc (DVDbuffer, DirPtr->CurrentBloc) < 0) 
					{
						free (DVDbuffer);
						return FS_ERROR_IO; 
					}
					
					Dir = (u8*) DVDbuffer; 
					DirPtr->PosInBloc = 0;
				}
				else 
				{
					free (DVDbuffer);
					return 0; 
				}
			}
		}	
		else { DirPtr->Info = (T_ISO9660_DR *)Dir; }
		
		if (((T_ISO9660_DR *)Dir)->DRsize != 0)
		{
			char * CharPtr;
			
			DirPtr->Info = (T_ISO9660_DR *) Dir;
			CharPtr = ((char*)DirPtr->Info) + sizeof(T_ISO9660_DR);
			
			// Convertion UNICODE --> ANSI
			if ((Encodage == ENCODAGE_Joliet) && ((T_ISO9660_DR *)Dir)->FIsize > 1)
			{
				char * ToConvert = ((char*)DirPtr->Info) + sizeof(T_ISO9660_DR);
				strncpy_unicode2ansi (ToConvert, ToConvert, ((T_ISO9660_DR *)Dir)->FIsize/2);
			}
			
			CharPtr = strstr (CharPtr, ";");
			if (CharPtr != NULL) { *CharPtr = 0x00; }
			free (DVDbuffer);		
			return 1;
		}
	} while (DirPtr->CurrentBloc < DirPtr->LastBloc);

	free (DVDbuffer);
	return 0;
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660_Internal::FindEntry (const char * Name, u32 * RetBlocId, u32 * RetBlocSize, u8 * RetFlags)
{
	ISO9660_DirPtr DirPtr;
	char * CurrentFilename = NULL;
	char FindFilename [ISO9660_MAX_FILENAME+1];
	char * BeginFilename;
	char * EndFilename;
	int	FilenameSize, RetVal, FilenameSize2;
	u32 BlocId;
	u32 BlocSize;
	u8 Found;	
	
	if (Name == NULL)		{ return ERROR_NULL_VALUE; }
	if (RootBlocId == 0)	{ return FS_ERROR_DEVICE_NOT_FOUND; }
	if (RootBlocSize == 0)	{ return FS_ERROR_DEVICE_NOT_FOUND; }
	
	BlocId	 	 = RootBlocId;
	BlocSize	 = RootBlocSize;

	*RetBlocId	 = 0;
	*RetBlocSize = 0;
	*RetFlags	 = 0;

	if (Name[0] == '/') { BeginFilename = (char*) Name+1; } 
	else { BeginFilename = (char*) Name; }
	memset (&FindFilename, 0x00, ISO9660_MAX_FILENAME+1);
	do
	{
		// Extraire nom du fichier/repertoire
		EndFilename = strstr (BeginFilename, "/");
		if (EndFilename == NULL) 
		{ 
			FilenameSize = strlen (BeginFilename);
			EndFilename = BeginFilename + FilenameSize;
		}
		else  
		{
			FilenameSize = EndFilename - BeginFilename; 
		}
		
		memcpy (&FindFilename, BeginFilename, FilenameSize);
		FindFilename [FilenameSize] = 0x00;
		Found = false;

	#if (ENTRY_BUFFER_SIZE > 0)	
		if (EntryBuffer != NULL)
		{
			char * TempString;
			
			if (EndFilename == NULL) { FilenameSize2 = strlen (Name); }
			else { FilenameSize2 = EndFilename - Name; }	
			TempString = (char*) malloc (FilenameSize2 + 1);
			if (TempString == NULL) break;
			memcpy (TempString, Name, FilenameSize2);
			TempString [FilenameSize2] = 0x00;
			
			for (int Boucle = 0; Boucle < ENTRY_BUFFER_SIZE; Boucle++)
			{
				if (EntryBuffer[Boucle].EntryName != NULL)
				{
					if (stricmp (EntryBuffer[Boucle].EntryName, TempString) == 0)
					{
						if (EndFilename[0] == 0x00) 
						{	
							*RetBlocId	 = EntryBuffer[Boucle].BlocId;
							*RetBlocSize = EntryBuffer[Boucle].DataLength;
							*RetFlags	 = EntryBuffer[Boucle].Fileflag;
							free (TempString);
							return NO_ERROR;
						}
						else
						{
							if (EntryBuffer[Boucle].Fileflag & ISO9660_DR_FLAG_DIRECTORY)
							{
								BlocId	 = (u32) EntryBuffer[Boucle].BlocId;
								BlocSize = (u32) EntryBuffer[Boucle].DataLength;
							}
						}
						
						Found = true;
					}
				}
			}
			
			free (TempString);
		}
	#endif		
		if (Found == false)
		{		
			OpenDir (BlocId, BlocSize, &DirPtr);
			
			do
			{
				RetVal = ReadDir (&DirPtr);
				
				switch (RetVal)
				{
					case 0:
						return FS_ERROR_NOT_FOUND;
						break;
					
					case 1:
						CurrentFilename = (char*) DirPtr.Info + sizeof(T_ISO9660_DR);
						if (stricmp (FindFilename, CurrentFilename) == 0) { Found = true; }
						break;
					
					default:
						return RetVal;
				}
				
			} while (Found == false);
			
		#if (ENTRY_BUFFER_SIZE > 0)			
			if (EntryBuffer != NULL)
			{
				if (EndFilename == NULL) { FilenameSize2 = strlen (Name); }
				else { FilenameSize2 = EndFilename - Name; }
				
				if (EntryBuffer[EntryBufferIndex].EntryName != NULL) { free(EntryBuffer[EntryBufferIndex].EntryName); }
				
/////////// BUGFIX : Plante avec chaine Homebrew si malloc < 16 ///////////////////////////////////////////////// 
//				EntryBuffer[EntryBufferIndex].EntryName = (char*) malloc (FilenameSize2+1);
				if (FilenameSize2 < 15) { EntryBuffer[EntryBufferIndex].EntryName = (char*) malloc (16); }
				else { EntryBuffer[EntryBufferIndex].EntryName = (char*) malloc (FilenameSize2+1); }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////				

				if (EntryBuffer[EntryBufferIndex].EntryName != NULL)
				{
					memcpy (EntryBuffer[EntryBufferIndex].EntryName,  Name, FilenameSize2);
					EntryBuffer[EntryBufferIndex].EntryName[FilenameSize2] = 0x00;
					EntryBuffer[EntryBufferIndex].BlocId		= DirPtr.Info->ExtentLoc.Value;
					EntryBuffer[EntryBufferIndex].DataLength	= DirPtr.Info->DataLength.Value;
					EntryBuffer[EntryBufferIndex].Fileflag		= DirPtr.Info->Fileflag	;	
					EntryBufferIndex++;
					if (EntryBufferIndex >= ENTRY_BUFFER_SIZE) { EntryBufferIndex = 0; }
				}	
			}
		#endif
			
			if (EndFilename[0] == 0x00) 
			{
				*RetBlocId	 = DirPtr.Info->ExtentLoc.Value;
				*RetBlocSize = DirPtr.Info->DataLength.Value;
				*RetFlags	 = DirPtr.Info->Fileflag;
				return NO_ERROR;
			}
			else
			{
				if (DirPtr.Info->Fileflag & ISO9660_DR_FLAG_DIRECTORY)
				{
					BlocId	 = (u32) DirPtr.Info->ExtentLoc.Value;
					BlocSize = (u32) DirPtr.Info->DataLength.Value;
				}
			}
		}
		
		BeginFilename = EndFilename + 1;
	} while (EndFilename != NULL);
	
	return FS_ERROR_NOT_FOUND;
}

///////////////////////////////////////////////////////////////////////////////
// Class C_FS_ISO9660                                                        //
///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::Init	() 
{
	RootBlocId		 = 0;
	RootBlocSize	 = 0;
	Encodage		 = 0;
	ObjectCount		 = 0;
	EntryBufferIndex = 0;

	return C_DVDx::Init ();  
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::Deinit () 
{
	RootBlocId		 = 0;
	RootBlocSize	 = 0;
	Encodage		 = 0;
	ObjectCount		 = 0;
	EntryBufferIndex = 0;
	
	return C_DVDx::Deinit ();
}

///////////////////////////////////////////////////////////////////////////////

C_FS_ISO9660::C_FS_ISO9660 () { }

///////////////////////////////////////////////////////////////////////////////

C_FS_ISO9660::~C_FS_ISO9660 () 
{
	Unmount();
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::Mount (const char *) 
{
	u32 RetVal;
	
	C_FileSystem::Mount();
	RetVal = C_DVDx::Mount();
	if (RetVal != NO_ERROR) 
	{
		C_FileSystem::Unmount(); 
		return RetVal;
	}
	
	RetVal = ReadHeader();
	if (RetVal != NO_ERROR) { C_FileSystem::Unmount(); }
	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::Unmount (void) 
{ 
	if (EntryBuffer != NULL)
	{
		for (int Boucle = 0; Boucle < ENTRY_BUFFER_SIZE; Boucle++)
		{
			EntryBuffer[Boucle].EntryName	= NULL;
			EntryBuffer[Boucle].BlocId		= 0;
			EntryBuffer[Boucle].Fileflag	= 0;
			EntryBuffer[Boucle].DataLength	= 0;
		}
	}
	
	return C_FileSystem::Unmount();
}

///////////////////////////////////////////////////////////////////////////////

C_File * C_FS_ISO9660::NewFile (void)
{
	return new C_FS_ISO9660::C_File();
}

///////////////////////////////////////////////////////////////////////////////

C_Dir * C_FS_ISO9660::NewDir (void)
{
	return new C_FS_ISO9660::C_Dir();
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::mkdir (const char * DirName)
{
	return FS_ERROR_READ_ONLY;
}

///////////////////////////////////////////////////////////////////////////////

T_String_PageCode C_FS_ISO9660::GetPageCode () 
{
	switch (Encodage) 
	{
		case ENCODAGE_ISO9660:	return String_PageCode_7bits; 
		case ENCODAGE_Joliet:	return String_PageCode_ANSI; 
	}
	
	return String_PageCode_None;
}

///////////////////////////////////////////////////////////////////////////////

const char * C_FS_ISO9660::GetStringParam ()
{
	return NULL;
}

///////////////////////////////////////////////////////////////////////////////

const char *  C_FS_ISO9660::GetFSname (void)
{
	static const char FSname[] = "dvd"; 
	
	return FSname;
}

///////////////////////////////////////////////////////////////////////////////
// Class C_FS_Fatlib::C_Dir                                                  //
///////////////////////////////////////////////////////////////////////////////

C_FS_ISO9660::C_Dir::C_Dir () 
{ 
	DirPtr.Info			= NULL;
	DirPtr.CurrentBloc	= 0;
	DirPtr.LastBloc		= 0;
	DirPtr.PosInBloc	= 0;
}

///////////////////////////////////////////////////////////////////////////////

C_FS_ISO9660::C_Dir::~C_Dir () { }

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::C_Dir::open (const char * Dirname)
{
	u8  RetFlags;
	int RetVal;
	
	FirstBlocId   = 0;
	FirstBlocSize = 0;
	RetFlags	  = 0;
	DirPtr.Info			= NULL;
	DirPtr.CurrentBloc	= 0;
	DirPtr.LastBloc		= 0;
	DirPtr.PosInBloc	= 0;
	Size = 0;
	if (V_Name == NULL) { return ERROR_OUT_OF_MEMORY; }
	RetVal = FindEntry (Dirname, &FirstBlocId, &FirstBlocSize, &RetFlags);
	if (RetVal != NO_ERROR) { return RetVal; }
	if (!(RetFlags & ISO9660_DR_FLAG_DIRECTORY)) { return FS_ERROR_NOT_FOUND; }
	RetVal = OpenDir (FirstBlocId, FirstBlocSize, &DirPtr);
	if (RetVal != NO_ERROR) { return RetVal; }
	return NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::C_Dir::close ()
{
	if (DirPtr.CurrentBloc == 0)	{ return FS_ERROR_NOT_OPENED; }
	DirPtr.CurrentBloc	= 0;
	FirstBlocId			= 0;
	FirstBlocSize		= 0;
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::C_Dir::next ()
{
	int RetVal;
	char * ChrPtr;
	
C_FS_ISO9660_C_Dir_Next_LOOP:
	Attrib = 0x00;
	V_Name [0] = 0x00;
	Size = 0;
	Name = V_Name;
	if (DirPtr.CurrentBloc == 0)	{ return FS_ERROR_NOT_OPENED; }
	if (DirPtr.CurrentBloc == 1)	{ return FS_EOF; }
	if (V_Name == NULL) 			{ return ERROR_OUT_OF_MEMORY; }
	
	RetVal = ReadDir (&DirPtr);
	
	if (RetVal < 0)		
	{ 
		DirPtr.CurrentBloc = 1;
		return RetVal;
	}
	
	if (RetVal == 0)
	{
		DirPtr.CurrentBloc = 1;
		return FS_EOF; 
	}
	
	if (DirPtr.Info == NULL) 
	{
		DirPtr.CurrentBloc = 1;
		return FS_EOF; 
	}
	
	
	if (DirPtr.Info->Fileflag & ISO9660_DR_FLAG_DIRECTORY) 
	{
		Attrib |= DIR_ATTRIB_DIRECTORY; 
	}
	else 
	{
		Size = DirPtr.Info->DataLength.Value; 
	}
	
	ChrPtr = (char*) DirPtr.Info + sizeof(T_ISO9660_DR);
	
	switch (ChrPtr[0])
	{
		case 0x00:
			strcpy (V_Name, ".");
			break;
		
		case 0x01:
			if (DirPtr.CurrentBloc == RootBlocId) { goto C_FS_ISO9660_C_Dir_Next_LOOP; }
			else { strcpy (V_Name, ".."); }
			break;
		
		default:
			strcpy (V_Name, ChrPtr);
			break;
	}
	
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::C_Dir::reset ()
{
	if (DirPtr.CurrentBloc == 0)	{ return FS_ERROR_NOT_OPENED; }
	if (V_Name == NULL) 			{ return ERROR_OUT_OF_MEMORY; }

	return OpenDir (FirstBlocId, FirstBlocSize, &DirPtr);
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::C_Dir::mkdir (const char * DirName)
{
	return FS_ERROR_READ_ONLY;
}

///////////////////////////////////////////////////////////////////////////////
// Class C_FS_Fatlib::C_File                                                 //
///////////////////////////////////////////////////////////////////////////////

C_FS_ISO9660::C_File::C_File (void)
{
	FileBlocId = 0;
	FileCurrentBloc = 0;
}

///////////////////////////////////////////////////////////////////////////////

C_FS_ISO9660::C_File::~C_File () { }

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::C_File::open (const char * Filename, u16 Mode)
{
	u8  RetFlags;
	int RetVal;
	
	if (FileBlocId != 0) { return FS_ERROR_FILE_OPENED; }
	FileBlocId  = 0;
	FileCurrentBloc = 0;
	FilePos  = 0;
	FileSize = 0;
	FilePos  = 0;
	
	if ((Mode & (FILE_OPEN_WRITE | FILE_OPEN_CREATE | FILE_OPEN_TRUNCATE)) != 0) 
	{ return FS_ERROR_READ_ONLY; }
	
	RetVal = FindEntry (Filename, &FileBlocId, &FileSize, &RetFlags);
	if (RetVal != NO_ERROR) 
	{
		FileBlocId  = 0;
		FileSize = 0;
		return RetVal; 
	}
	
	if (RetFlags & ISO9660_DR_FLAG_DIRECTORY) 
	{
		FileBlocId  = 0;
		FileSize = 0;
		return FS_ERROR_NOT_FILE; 
	}

	FileCurrentBloc = FileBlocId;
	if ((Mode & FILE_OPEN_APPEND) == FILE_OPEN_APPEND) { this->seek (FILE_SEEK_END, 0); }
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::C_File::close (void)
{	
	if (FileSize == 0) { return FS_ERROR_NOT_OPENED; }
	
	FileBlocId  = 0;
	FileSize = 0;
	FilePos  = 0;
	FileCurrentBloc = 0;
	FilePos = 0;
	
	return NO_ERROR;
} 

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::C_File::seek (u8 Mode, u32 MyPos)
{
	if (FileSize == 0) { return FS_ERROR_NOT_OPENED; }
	
	switch (Mode)
	{
		case FILE_SEEK_SET:
			if (MyPos > FileSize) { return ERROR_BAD_PARAM; }
			FilePos = MyPos;
			FileCurrentBloc = FileBlocId + (MyPos / ISO9660_BLOC_SIZE);
			break;
		
		case FILE_SEEK_CUR:
			if (FilePos+MyPos > FileSize) { return ERROR_BAD_PARAM; }
			FilePos += MyPos;
			FileCurrentBloc = FileBlocId + (MyPos / ISO9660_BLOC_SIZE);
			break;
		
		case FILE_SEEK_END:
			if (MyPos > FileSize) { return ERROR_BAD_PARAM; }
			FilePos = FileSize - MyPos;
			FileCurrentBloc = FileBlocId + (MyPos / ISO9660_BLOC_SIZE);
			break;
		
		default:
			return ERROR_BAD_PARAM;
	}
	
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

s32 C_FS_ISO9660::C_File::read (void * Buffer, s32 Size)
{
	u32 BufferPos;
	u8 * DVDbuffer;

	if (Buffer  == NULL)				{ return ERROR_NULL_VALUE; }
	if (FileSize == 0)					{ return 0; }
	if (FilePos >= FileSize)	{ return 0; }
	
	DVDbuffer = (u8*) malloc(ISO9660_BLOC_SIZE);
	if (DVDbuffer == NULL) { return ERROR_OUT_OF_MEMORY; }
	
C_FS_ISO9660_C_File_read_LOOP:	
	if (C_DVDx::ReadBloc (DVDbuffer, FileCurrentBloc) < 0) 
	{
		free (DVDbuffer);
		return FS_ERROR_IO; 
	}

	if (FilePos + Size > FileSize)
	{
		Size = FileSize - FilePos;
	}
	
	BufferPos = FilePos % ISO9660_BLOC_SIZE;
	if (BufferPos + Size > ISO9660_BLOC_SIZE)
	{
		u32 BufferSize;
		char * CharBuffer;
		
		BufferSize = ISO9660_BLOC_SIZE - BufferPos;
		memcpy (Buffer, DVDbuffer+BufferPos, BufferSize);
		CharBuffer = (char*)Buffer;
		CharBuffer += BufferSize;
		Buffer = CharBuffer;
		FilePos += BufferSize;
		Size -= BufferSize;
		FileCurrentBloc++;
		goto C_FS_ISO9660_C_File_read_LOOP;
	}
	else
	{
		memcpy (Buffer, DVDbuffer+BufferPos, Size);
		FilePos += Size;
	}
	
	FileCurrentBloc = FileBlocId + (FilePos / ISO9660_BLOC_SIZE);
	free (DVDbuffer);
	return Size;
}

///////////////////////////////////////////////////////////////////////////////

s32 C_FS_ISO9660::C_File::write (const void * Buffer, s32 Size)
{
	return FS_ERROR_READ_ONLY; 
}

///////////////////////////////////////////////////////////////////////////////

int C_FS_ISO9660::C_File::eof(void)
{
	if (FileSize == 0) { return FS_ERROR_NOT_OPENED; }
	if (FilePos >= FileSize)	{ return true; }
	return false;
}

///////////////////////////////////////////////////////////////////////////////
