///////////////////////////////////////////////////////////////////////////////
// Classes C_FS_ISO9660, C_FS_ISO9660::C_File and C_FS_ISO9660::C_Dir        //
// Created by Hell Hibou (2009)                                              //
// Part of MyFS library                                                      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#ifndef _CLASS_UOBIHLLEH_C_FS_ISO9660
#define _CLASS_UOBIHLLEH_C_FS_ISO9660

#include "FS.hpp"
#include "T_ISO9660.h"

class C_FS_ISO9660_Internal
{
	protected:
		typedef struct T_EntryBuffer
		{
			char *				EntryName;
			u32		BlocId;
			u32		DataLength;
			u32		Fileflag;
		} T_EntryBuffer;
		
		static C_FS_ISO9660_Internal::T_EntryBuffer * EntryBuffer;
		static u32	ObjectCount;
		static u16  EntryBufferIndex;
		static u32	RootBlocId;
		static u32	RootBlocSize;
		static u8	Encodage;
		
		C_FS_ISO9660_Internal();
		~C_FS_ISO9660_Internal();
		int ReadHeader ();
		int OpenDir (u32 BlocId, u32 Size, ISO9660_DirPtr * DirPtr);
		int ReadDir (ISO9660_DirPtr * DirPtr);
		int FindEntry (const char * Name, u32 * RetBlocId, u32 * RetBlocSize, u8 * RetFlags);
};

class C_FS_ISO9660: public C_FileSystem, protected C_FS_ISO9660_Internal
{
	public:	
		static int Init ();
		static int Deinit ();
		
		C_FS_ISO9660  (void);
		~C_FS_ISO9660 (void);
		int Mount     (const char * = NULL);
		int Unmount   (void);
		int mkdir     (const char *);
		::C_File    * NewFile (void);
		::C_Dir     * NewDir  (void);
		const char * GetFSname (void);
		const char * GetStringParam	(void);
		T_String_PageCode GetPageCode (void);
	
	/////////////////////////////////

	protected:
		class C_Dir: public ::C_Dir, protected C_FS_ISO9660_Internal
		{
			public:
				C_Dir     ();
				~C_Dir    (void);
				int open  (const char *);
				int close ();
				int	next  ();
				int reset ();
				int mkdir (const char *);
			
			private:
				ISO9660_DirPtr DirPtr;
				u32 FirstBlocId;
				u32 FirstBlocSize;
		};
	
	/////////////////////////////////
	
	protected:
		
		class C_File: public ::C_File, protected C_FS_ISO9660_Internal
		{
			public:			
				C_File    ();
				~C_File   (void);
				int open  (const char *, u16 Mode);
				int close ();
				int seek  (u8 Mode, u32 Pos);
				s32 read  (void * Buffer, s32 Size);
				s32 write (const void * Buffer, s32 Size);
				int eof ();
			
			private:
				u32 FileBlocId;
				u32 FileCurrentBloc;
		};
};

#endif

///////////////////////////////////////////////////////////////////////////////
