///////////////////////////////////////////////////////////////////////////////
// Classes C_FS_SMB, C_FS_SMB::C_File and C_FS_SMB::C_Dir                    //
// Created by Hell Hibou (2009)                                              //
// Part of MyFS library                                                      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#ifndef _CLASS_UOBIHLLEH_C_FS_SMB
#define _CLASS_UOBIHLLEH_C_FS_SMB

#include "FS.hpp"
#include <smb.h>


class C_FS_SMB: public C_FileSystem
{
	public:	
		SMBCONN Connection;
		
		static int NetworkInit ();
		
		C_FS_SMB    ();
		C_FS_SMB    (const char * ServerIP, const char * Share, const char * User = "", const char * Password = "");
		~C_FS_SMB   (void);
		int Mount   (const char * = NULL);
		int Unmount (void);
		int mkdir   (const char *);
		::C_File * NewFile (void);
		::C_Dir  * NewDir  (void);
		
		const char * GetStringParam	(void);
		const char * GetFSname  (void);
		T_String_PageCode GetPageCode (void);
	
	protected:
		int CheckMount (void);

	private:
		typedef struct T_ConnectSMB
		{
			u8 Status;
			SMBCONN * Connection;
			const char * ServerIP;
			const char * Share;
			const char * User;
			const char * Password;
		} T_ConnectSMB;
		
		static u8	isInit;
		u8			Connected;
		char *		FSname;
		
		int    		  Connect (const char *);
		static void * Process_ConnectSMB (void * arg);
		static int    Init();
		static char * ConvertPath (const char *);

	/////////////////////////////////
	
	public:
		class C_Dir: public ::C_Dir
		{
			public:
				C_Dir     (C_FS_SMB *);
				~C_Dir    (void);
				int open  (const char *);
				int close ();
				int	next  ();
				int reset ();
				int mkdir (const char *);
			
			private:
				C_FS_SMB *  FS;
				char * 		CurrentDir;
				u8        	DirConnected;
				u32			DirId;
		};
		
		/////////////////////////////////
		
		class C_File: public ::C_File
		{
			public:			
				C_File    (C_FS_SMB *);
				~C_File   (void);
				int open  (const char *, u16 Mode);
				int close ();
				int seek  (u8 Mode, u32 Pos);
				s32 read  (void * Buffer, s32 Size);
				s32 write (const void * Buffer, s32 Size);
				int eof   ();
			
			private:
				C_FS_SMB *	FS;
				SMBFILE		FilePtr;
		};
};

#endif

///////////////////////////////////////////////////////////////////////////////
