///////////////////////////////////////////////////////////////////////////////
// Classes C_FS_Fatlib, C_FS_Fatlib::C_File and C_FS_Fatlib::C_Dir           //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Valid PARTITION_INTERFACE values: PI_SDGECKO_A   for SD Gecko in slot A   //
//                                   PI_SDGECKO_B   for SD Gecko in slot B   //
//                                   PI_INTERNAL_SD for front SD             //
//                                   PI_USBSTORAGE  for Flash USB            //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#ifndef _CLASS_UOBIHLLEH_C_FS_FATLIB
#define _CLASS_UOBIHLLEH_C_FS_FATLIB

#include <fat.h>
#include <sys/dir.h>
#include "FS.hpp"

class C_FS_Fatlib: public C_FileSystem
{
	public:
		C_FS_Fatlib  (PARTITION_INTERFACE);
		~C_FS_Fatlib (void);
		int Mount    (void);
		int Unmount  (void);
		::C_File    * NewFile (void);
		::C_Dir     * NewDir  (void);
		int mkdir (const char *);
		
		T_String_PageCode GetPageCode (void);
		const char * GetFSname	(void);
		const char * GetStringParam	(void);
		

	private:
		static u8 isInit;
		
		PARTITION_INTERFACE DriveId;
		u8 isMounted;

	protected:
		class C_Dir: public ::C_Dir
		{
			public:
				C_Dir    (C_FS_Fatlib *);
				~C_Dir   (void);
				int open (const char *);
				int close ();
				int	next ();
				int reset ();
				int mkdir (const char *);
			
			private:
				struct stat st;
				DIR_ITER *	Dir;
				C_FS_Fatlib * FS;
		};
		
		/////////////////////////////////
		
		class C_File: public ::C_File
		{
			public:			
				C_File    (C_FS_Fatlib *);
				~C_File   (void);
				int open  (const char *, u16 Mode);
				int close ();
				int seek  (u8 Mode, u32 Pos);
				s32 read  (void * Buffer, s32 Size);
				s32 write (const void * Buffer, s32 Size);
				int eof ();
			
			private:
				FILE * FilePtr;
				C_FS_Fatlib * FS;
		};
		
};

#endif

///////////////////////////////////////////////////////////////////////////////
