///////////////////////////////////////////////////////////////////////////////
// ISO9660 structures                                                        //
// Created by Hell Hibou (2009)                                              //
// Part of MyFS library                                                      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#ifndef _T_UOBIHLLEH_ISO9660
#define _T_UOBIHLLEH_ISO9660

//////////////////////////////////////////////////////////////////////////////
// Basic data definition                                                    //
//////////////////////////////////////////////////////////////////////////////

#include <gccore.h>

#define ENDIAN_NOTREVERSE /* Ne pas inverser les octes */

#ifndef PACKED
#	define PACKED __attribute__((__packed__))
#endif

#define U8	u8
#define S8	s8
#define U16	u16
#define U32	u32

#ifdef ENDIAN_NOTREVERSE
typedef struct T_BothEndianWord
{
	U16 Reverse;
	U16 Value;
} T_BothEndianWord;
#else
typedef struct T_BothEndianWord
{
	U16 Value;
	U16 Reverse;
} T_BothEndianWord;
#endif


#ifdef ENDIAN_NOTREVERSE
typedef struct T_BothEndianDWord
{
	U32 Reverse;
	U32 Value;
} T_BothEndianDWord;
#else
typedef struct T_BothEndianDWord
{
	U32 Value;
	U32 Reverse;
} T_BothEndianDWord;
#endif

//////////////////////////////////////////////////////////////////////////////
// Format of a Directory Record                                             //
//////////////////////////////////////////////////////////////////////////////

#define ISO9660_BLOC_SIZE				2048
#define ISO9660_DR_FLAG_EXIST			0x01
#define ISO9660_DR_FLAG_DIRECTORY		0x02
#define ISO9660_DR_FLAG_ASSOCIATEDFILE	0x04
#define ISO9660_DR_FLAG_RECORD			0x08
#define ISO9660_DR_FLAG_PROTECTION		0x10
#define ISO9660_DR_FLAG_MULTIEXTEND		0x80

#define ISO9660_ESCAPESEQUENCE_JOLIET_LEVEL1 "%/@"
#define ISO9660_ESCAPESEQUENCE_JOLIET_LEVEL2 "%/C"
#define ISO9660_ESCAPESEQUENCE_JOLIET_LEVEL3 "%/E"

typedef struct T_ISO9660_DR_DateTime
{
	U8 Year;
	U8 Month;
	U8 Day;
	U8 Houre;
	U8 Minute;
	U8 Sectond;
	U8 Offset; // Offset from Greenwich Mean Time in number of 15 min intervals
} PACKED T_ISO9660_DR_DateTime;

typedef struct T_ISO9660_DR
{
	U8	DRsize;							// Length of Directory Record
	U8	EARsize;						// Extended Attribute Record Length
	T_BothEndianDWord		ExtentLoc;	// Location of Extent est le numro de secteur du rpertoire principal
  	T_BothEndianDWord		DataLength;	// DataLength nous donne le nombre d'octets de ce rpertoire. En divisant par 2048 nous obtenons le nombre de secteurs qu'il utilise.
	T_ISO9660_DR_DateTime	DateTime;	// Recording Date and Time
	U8						Fileflag;
	U8						FUsize;		// File Unit Size
	U8						IGsize;		// Interleave Gap Size
	T_BothEndianWord		VSnumber;	// Volume Sequence Number
	U8						FIsize;		// Length of File Identifier
	
	// File Identifier,	size = FIsize
	// Padding Field,	size = U8, value = 0x00
	// System use,		size = the rest of the bloc
} PACKED T_ISO9660_DR;

typedef struct T_ISO9660_RootDR
{
	U8	DRsize;							// Length of Directory Record
	U8	EARsize;						// Extended Attribute Record Length
	T_BothEndianDWord		ExtentLoc;	// Location of Extent est le numro de secteur du rpertoire principal
  	T_BothEndianDWord		DataLength;	// DataLength nous donne le nombre d'octets de ce rpertoire. En divisant par 2048 nous obtenons le nombre de secteurs qu'il utilise.
	T_ISO9660_DR_DateTime	DateTime;	// Recording Date and Time
	U8						Fileflag;
	U8						FUsize;		// File Unit Size
	U8						IGsize;		// Interleave Gap Size
	T_BothEndianWord		VSnumber;	// Volume Sequence Number
	U8						FIsize;		// Length of File Identifier
	U8		 				Padding;	// = 0x00
} PACKED T_ISO9660_RootDR;


//////////////////////////////////////////////////////////////////////////////
// Volume Descriptor                                                        //
//////////////////////////////////////////////////////////////////////////////

#define ISO9660_VD_LBA_BEGIN	16
#define ISO9660_VD_LBA_END		31
#define ISO9660_VD_MAGIC		"CD001"
#define ISO9660_VD_MAGIC_SIZE	5

#define ISO9660_MAX_FILENAME	255

enum T_ISO9660_VD_VDT
{
	ISO9660_VD_Boot = 0,	// Volume Descriptor is a Boot Record
	ISO9660_VD_PVD,			// Volume Descriptor is a Primary Volume Descriptor
	ISO9660_VD_SVD,			// Volume Descriptor is a Supplementary Volume Descriptor
	ISO9660_VD_VPD,			// Volume Descriptor is a Volume Partition Descriptor
	ISO9660_VD_VDST = 0xFF	// Volume Descriptor is a Volume Descriptor Set Terminator
};

typedef struct T_ISO9660_VD_DateTime
{
	char Year	[4];
	char Month	[2];
	char Day	[2];
	char Houre	[2];
	char Minute	[2];
	char Second	[2];
	char Hsecond[2];
	S8	 Offest; // Offset from Greenwich Mean Time in number of 15 min intervals
} PACKED T_ISO9660_VD_DateTime;

typedef struct T_ISO9660_VD_Boot
{
	char SystemIdentifier	[32];
	char BootIdentifier		[32];
	U8   Bin				[1977];
} PACKED T_ISO9660_VD_Boot;


typedef struct T_ISO9660_VD_xVD
{ 
	U8						VolumeFlags;	// = 0x00 for PVD
	char					SystemIdentifier	[32];
	char					VolumeIdentifier	[32];
	U8						Unused_1			[8];	// = 0x00
	T_BothEndianDWord		VolumeSpaceSize;
	char					EscapeSequences		[32];	// = 0x00 for PVD
	T_BothEndianWord		VolumeSetSize;
	T_BothEndianWord		VolumeSequenceNumbre;
	T_BothEndianWord		LBS;
	T_BothEndianDWord		PathTableSize;
	u32						LpathTable;				// Location of Occurrence of Type L Path Table
	u32						LpathTableOpt;			// Location of Optional Occurrence of Type L Path Table
	u32						MpathTable;				// Location of Occurrence of Type M Path Table
	u32						MpathTableOpt;			// Location of Optional Occurrence of Type M Path Table
	T_ISO9660_RootDR 		RootDir;				// Directory Record for Root Directory
	char					VolumeSetId			[128];
	char					PublisherIde		[128];
	char					DataPreparerId		[128];
	char					ApplicationId		[128];
	char					CopyrightFileId		[37];
	char					AbstractFileId		[37];
	char					BibliographicFileId	[37];
	T_ISO9660_VD_DateTime	VolumeCreation;			// Volume Creation Date and Time
	T_ISO9660_VD_DateTime	VolumeCModification;	// Volume Modification Date and Time
	T_ISO9660_VD_DateTime	VolumeExpiration;		// Volume Expiration Date and Time
	T_ISO9660_VD_DateTime	VolumeEffective;		// Volume Effective Date and Time
	U8						FSversion;				// File Structure Version
	U8						Reserved_1; 			// = 0x00
	U8						ApplicationUse	[512];	// ?? = 0x00
	U8						Reserved_2		[653];	// = 0x00
} PACKED T_ISO9660_VD_xVD;


typedef struct T_ISO9660_VD_VPD
{
	U8						Unused_1;
	char					SystemIdentifier	[32];
	char					VPI	[32];	// Volume Partition Identifier
	T_BothEndianDWord		VPL;		// Volume Partition Location
	T_BothEndianDWord		VPS;		// Volume Partition Size
	U8						Bin [1960];	// System Use
} PACKED T_ISO9660_VD_VPD;

typedef struct T_ISO9660_VD 
{
	U8	Type;
	char Magic [5];
	U8	Version;
	union 
	{
		U8 VDST [2041];		// End of Volume Descriptor, set to 0
		T_ISO9660_VD_Boot	Boot;
		T_ISO9660_VD_xVD	PVD;
		T_ISO9660_VD_xVD	SVD;
		T_ISO9660_VD_VPD	VPD;
	} PACKED VD;

} PACKED T_ISO9660_VD;


//////////////////////////////////////////////////////////////////////////////

typedef struct ISO9660_DirPtr
{
	u32 CurrentBloc;
	u32 LastBloc;
	u32 PosInBloc;
	T_ISO9660_DR * Info;
} ISO9660_DirPtr;

//////////////////////////////////////////////////////////////////////////////

#endif
