///////////////////////////////////////////////////////////////////////////////
// Class C_Interface                                                         //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <string.h>
#include <sys/dir.h>

#include "kmemory.h"
#include "Ressources.h"
#include "system.h"
#include "AppConfig.h"

#include "Interface.hpp"
#include "control.hpp"

///////////////////////////////////////////////////////////////////////////////

#define REDRAW			(0x03)
#define REDRAW_NONE		(0x00)
#define REDRAW_MENU		(0x01)
#define REDRAW_ALL  	(0x03)

u8 C_Interface::ExitCode = 0;
u8 C_Interface::BlocRedraw = 0;
C_Menu * C_Interface::Menu = NULL;
C_Control::T_Control C_Interface::V_NULL = { 0, NULL, 0, 0};

T_RGB C_Interface::ColorTextBar			= RGB_GREY;
T_RGB C_Interface::ColorTextChannel		= RGB_GREY;
T_RGB C_Interface::ColorTextMessageBox	= RGB_GREY;

extern void BeforeExit ();

///////////////////////////////////////////////////////////////////////////////
// Fonctions public                                                          //
///////////////////////////////////////////////////////////////////////////////

C_Interface::C_Interface()
{
	Channels_PaddingX 	= 40;
	Channels_PaddingY 	= 30;
	Channels_SpaceX   	= 77;
	Channels_SpaceY   	= 120;
	BlocRedraw			= 0;
	ChannelBlocId 		= 0;
	NbChannels 			= MAX_CHANNELS;
	ChannelNbNavi 		= 0;
	ChannelSelected_X 	= 0;
	ChannelSelected_Y 	= 0;
	HaveRedraw 			= REDRAW_ALL;
	RedrawCount 		= 0;
	ChannelSelected		= 0;
	TextColor			= ColorTextChannel;
	EmptyMsg			= NULL;
	ControlAllowed		= INTERFACE_CONTROL_ALLOW_ALL;
	
	TempImage.Height	= 95;
	TempImage.Width		= 132;
	TempImage.NbImages	= 1;
	
	TempImage.Image = (T_RGBA*) MEM2_malloc(sizeof(T_RGBA)*95*132);
	Channel = (T_Channel*) MEM2_malloc (sizeof(T_Channel) * MAX_CHANNELS);
	
	for (int Boucle = 0; Boucle < MAX_CHANNELS; Boucle++)
	{
		Channel[Boucle].Icon.Image		= NULL;
		Channel[Boucle].Icon.Width		= 0;
		Channel[Boucle].Icon.Height		= 0;
		Channel[Boucle].Icon.NbImages	= 0;
		Channel[Boucle].IconId 			= 0;
		Channel[Boucle].FadeId			= 0;
		Channel[Boucle].Flags	 		= 0;
		memset (Channel[Boucle].Name, 0x00, C_Interface_MAX_NAME_SIZE);
	}
}

///////////////////////////////////////////////////////////////////////////////

C_Interface::~C_Interface (void)
{
	int Boucle;
	
	for (Boucle = 0; Boucle < MAX_CHANNELS; Boucle++)
	{
		if (Channel[Boucle].Icon.Image != NULL) 
		{ MEM2_free(Channel[Boucle].Icon.Image); }
	}

	if (Channel != NULL) MEM2_free(Channel);
	if (TempImage.Image != NULL) MEM2_free(TempImage.Image);
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface::Init()
{
	if(Channel == NULL) { return; }
	Clean();
	Read();
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface::SetRedrawAll()
{
	HaveRedraw = REDRAW_ALL;
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface::SetNoRedraw()
{
	HaveRedraw = REDRAW_NONE;
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface::PostRetraceCallback (u32 Count)
{
	u32 Boucle;

	if (Menu != NULL)
	{
		if (BlocRedraw == 1)
		{
			C_GFX::PostRetraceCallback (Count); return; 
		}
		
		Menu->PostRetraceCallback(Count);
		return;
	}
	
	if (Channel == NULL) { return; }
	C_GFX::PostRetraceCallback (Count);
	if (BlocRedraw == 1) { return; }	
	RedrawCount++;

	if (RedrawCount >= 500)
	{
		RedrawCount = 0;
		for (Boucle = 0; Boucle < MAX_CHANNELS; Boucle++)
		{
			if (Channel[Boucle].Icon.NbImages > 1)
			{
				Channel[Boucle].FadeId = 0;
				Channel[Boucle].Flags |= INTERFACE_FLAG_USEFADE;
				HaveRedraw = REDRAW_ALL;
			}
		}
	}
	else
	{
		for (Boucle = 0; Boucle < MAX_CHANNELS; Boucle++)
		{
			if (Channel[Boucle].Flags & INTERFACE_FLAG_USEFADE)
			{ 
				HaveRedraw = REDRAW_ALL; 
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////

int C_Interface::Show (C_Control::T_Control Control)
{

	switch (ExitCode)
	{
		case EXITCODE_WIIMENU:
			BeforeExit();
			WII_Initialize();
			WII_ReturnToMenu();
			break;
		
		case EXITCODE_SCHUTDOWN:
			BeforeExit();
			SetPowerMode (WiiPower_Shutdown);
			break;
	}	
	
	if (BlocRedraw == 1) { return NO_ERROR; }
	if (ChannelSelected > NbChannels) { ChannelSelected = 0; }
	ChannelSelected_Y = ChannelSelected /  CHANNELS_BY_LINE;
	ChannelSelected_X = ChannelSelected - (ChannelSelected_Y * CHANNELS_BY_LINE);

	if (Control.PAD & CONTROL_START_HOME)
	{
		Menu = new C_Menu();
		Menu->Show();
		Control = C_Control::Read(0);
		delete (Menu);
		Menu = NULL;
		this->SetRedrawAll();
	}
	else if (Control.PAD & CONTROL_A)
	{
		int ChannelId = ChannelSelected_X + (ChannelSelected_Y * CHANNELS_BY_LINE);
		
		if (Channel[ChannelId].Name[0] != 0x00)
		{			
			int RetVal;
			
			C_GFX::Begin ();
			HaveRedraw = REDRAW_ALL;
			DrawChannels(false);
			C_GFX::End();		
			
			RetVal = this->Run(ChannelId);
			if (RetVal < 0)
			{
				ErrorMessageBox (RetVal);
				C_GFX::Begin ();
				HaveRedraw = REDRAW_ALL;
				DrawChannels(false);
				C_GFX::End(); 
			}
			
			Control = C_Control::Read(0);
		}
	}
	else if (Control.PAD & CONTROL_RIGHT)
	{
		if ( (ChannelSelected_X < (CHANNELS_BY_LINE - 1)) && (Channel[ChannelSelected_Y*CHANNELS_BY_LINE + ChannelSelected_X + 1].Name [0] != 0x00) )
		{
			ChannelSelected_X++;
			HaveRedraw = REDRAW_ALL;
		}
	}
	else if (Control.PAD & CONTROL_LEFT)
	{
		if (ChannelSelected_X > 0)
		{
			ChannelSelected_X--;
			HaveRedraw = REDRAW_ALL;
		}
	}
	else if (Control.PAD & CONTROL_DOWN)
	{
		if ( (ChannelSelected_Y < (CHANNELS_BY_COL - 1)) && (Channel[(ChannelSelected_Y+1)*CHANNELS_BY_LINE + ChannelSelected_X].Name[0] != 0x00) )
		{
			ChannelSelected_Y++;
			HaveRedraw = REDRAW_ALL;
		}
	}
	else if (Control.PAD & CONTROL_UP)
	{
		if (ChannelSelected_Y > 0)
		{
			ChannelSelected_Y--;
			HaveRedraw = REDRAW_ALL;
		}
	}
	else if (Control.PAD & CONTROL_R_PLUS) // Chaines suivantes
	{
		if ((ChannelNbNavi&2) == 2)
		{
			C_GFX::Begin ();
				HaveRedraw = REDRAW_ALL;
				DrawChannels(false);
			C_GFX::End();
			ChannelBlocId += MAX_CHANNELS;
			Read ();
			HaveRedraw = REDRAW_ALL;
			Control = C_Control::Read(0);
		}
	}
	else if (Control.PAD & CONTROL_L_MINUS) // Chaines precedentes
	{
		if (ChannelBlocId > 0)
		{
			C_GFX::Begin ();
				HaveRedraw = REDRAW_ALL;
				DrawChannels(false);
			C_GFX::End();
			ChannelBlocId -= MAX_CHANNELS;
			Read ();
			HaveRedraw = REDRAW_ALL;
			Control = C_Control::Read(0);
		}
	}
	else if ((Control.PAD & CONTROL_Z_1) && (ControlAllowed & INTERFACE_CONTROL_ALLOW_REFRESH)) // Rafraichissement des chaines
	{
		C_GFX::Begin ();
			HaveRedraw = REDRAW_ALL;
			DrawChannels(false);
		C_GFX::End();
		ChannelBlocId = 0;
		Read ();
		HaveRedraw = REDRAW_ALL;
		Control = C_Control::Read(0);
	}
	
	ChannelSelected = ChannelSelected_X + (ChannelSelected_Y * CHANNELS_BY_LINE);
	
	// Pointeur Infra-Rouge de la Wiimote
#ifdef ENABLE_IR_CURSOR
	if (Control.IrValid & CONTROL_VALID)
	{
		int X, Y;
		
		for (Y = (CHANNELS_BY_COL - 1); Y > -1; Y--)
		{
			for (X = (CHANNELS_BY_LINE - 1); X > -1; X--)
			{
				int ChannelId = X + (Y * CHANNELS_BY_LINE);
				
				if (Channel[ChannelId].Name[0] != 0x00)
				{
					if ((Control.IrX >=  (X*Channels_SpaceX+Channels_PaddingX)) && 
					    (Control.IrX <= ((X*Channels_SpaceX+Channels_PaddingX) + ImageRessource[ImageRessource_ThumbDefault].Width)) &&
				        (Control.IrY >=  (Y*Channels_SpaceY+Channels_PaddingY)) &&
				        (Control.IrY <=  (Y*Channels_SpaceY+Channels_PaddingY) + ImageRessource[ImageRessource_ThumbDefault].Height) ) 
					{
						if ((ChannelSelected_X != X) || (ChannelSelected_Y != Y))
						{
							HaveRedraw = REDRAW_ALL;
						}
						
						ChannelSelected_X = X;
						ChannelSelected_Y = Y;
						ChannelSelected = ChannelSelected_X + (ChannelSelected_Y * CHANNELS_BY_LINE);
					}
				}
			}
		}	  
	}
#endif

	if (HaveRedraw & REDRAW) 
	{
		C_GFX::Begin ();
		DrawChannels(true);	
		C_GFX::Flush();
	}
	
#ifdef ENABLE_IR_CURSOR	
	if (Control.IrValid)
	{ 
		if (!(HaveRedraw & REDRAW))
		{
			C_GFX::Begin ();
			C_GFX::Flush ();
		}
		
		if (Control.IrValid & CONTROL_VALID) { C_GFX::ShowImage (ImageRessource[ImageRessource_Cursor], Control.IrX, Control.IrY, 0); }
	}
#endif
	
	if ((HaveRedraw & REDRAW) || (Control.IrValid))
	{
		HaveRedraw = REDRAW_NONE; 
		C_GFX::End ();
	}
	
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface::MessageBox (const char * Message, u8 Type)
{
	u32 MessageSize;
	
	if (Type == MESSAGEBOX_HIDE)
	{
		BlocRedraw = 0;
		if (Menu != NULL) { Menu->BlocRedraw = 0; }
		return;
	}
	
	BlocRedraw = 1;
	if (Menu != NULL) { Menu->BlocRedraw = 1; }
	C_GFX::Begin ();
	C_GFX::ShowImage (ImageRessource[ImageRessource_MessageBox], (C_GFX::Width/2) - (ImageRessource[ImageRessource_MessageBox].Width/2), (C_GFX::Height/2) - (ImageRessource[ImageRessource_MessageBox].Height/2), 0);

	MessageSize = C_GFX::SizeOfString (Message); 
	if (MessageSize-20 < ImageRessource[ImageRessource_MessageBox].Width)
	{
		C_GFX::ShowString (Message, (C_GFX::Width/2) - (MessageSize/2), (C_GFX::Height/2) - (ImageRessource[ImageRessource_Font].Height/2), ColorTextMessageBox);
	}
	else
	{
		char * MyMessage;
		char * MessageBegin;
		char * MessageEnd;
		u32 TotalStringSize, StringSize, Line;
		
		MyMessage = (char *) MEM2_malloc (strlen(Message+1));
		strcpy (MyMessage, Message);
		MessageBegin = MyMessage;
		Line = 0;
		TotalStringSize = 0;
		
		do 
		{
			
			MessageEnd = strstr (MessageBegin, " ");
			if (MessageEnd != NULL) { MessageEnd[0] = 0x00; }
			StringSize = C_GFX::SizeOfString (MessageBegin);
			
			if (StringSize + 20 + TotalStringSize <= ImageRessource[ImageRessource_MessageBox].Width)
			{
				StringSize += ImageRessource[ImageRessource_Font].Width/2;
				C_GFX::ShowString (MessageBegin, (C_GFX::Width/2) - (ImageRessource[ImageRessource_MessageBox].Width/2) + 10 + TotalStringSize, (C_GFX::Height/2) - (ImageRessource[ImageRessource_MessageBox].Height/2) + 15 + (ImageRessource[ImageRessource_Font].Height * Line), ColorTextMessageBox);
				TotalStringSize += StringSize;	
			}
			else
			{
				Line++;
				C_GFX::ShowString (MessageBegin, (C_GFX::Width/2) - (ImageRessource[ImageRessource_MessageBox].Width/2) + 10, (C_GFX::Height/2) - (ImageRessource[ImageRessource_MessageBox].Height/2) + 15 + (ImageRessource[ImageRessource_Font].Height * Line), ColorTextMessageBox);
				TotalStringSize = StringSize + ImageRessource[ImageRessource_Font].Width/2;
			}
			
			if (MessageEnd != NULL) { MessageBegin = MessageEnd + 1; }
			
		} while (MessageEnd != NULL);
		
		MEM2_free (MyMessage);
	}

	if (Type != MESSAGEBOX_NOWAIT)	
	{
		MessageSize = C_GFX::SizeOfString (textRessource[TextRessource_PressA]); 
		C_GFX::ShowString (textRessource[TextRessource_PressA], (C_GFX::Width/2) - (MessageSize/2), (C_GFX::Height/2) - (ImageRessource[ImageRessource_Font].Height) + (ImageRessource[ImageRessource_MessageBox].Height/2) - 15, ColorTextMessageBox);
	}
	
	C_GFX::Flush();
	C_GFX::End();

	if (Type != MESSAGEBOX_NOWAIT)
	{
		while ((C_Control::Read(0).PAD & CONTROL_A) != CONTROL_A);
		while ((C_Control::Read(0).PAD & CONTROL_A) == CONTROL_A);
	}
	
	if (Type != MESSAGEBOX_NOWAIT)	
	{
		BlocRedraw = 0; 
		if (Menu != NULL)
		{
			Menu->BlocRedraw = 0;
			Menu->DrawMenu ();
			Menu->DrawMenu ();
		}
		else { this->SetRedrawAll(); }
	}
};

///////////////////////////////////////////////////////////////////////////////

void C_Interface::ErrorMessageBox (const int Code)
{
	int Boucle = 0;
	
	switch (Code)
	{
		case -ENOEXEC:
			MessageBox (textRessource[TextRessource_InvalidExe], MESSAGEBOX_OK);
			break;
		
		case -10015:
			MessageBox (textRessource[TextRessource_ConnexionError], MESSAGEBOX_OK);
			break;
			
		default:
			while ( (TextErrors[Boucle].Message != NULL) && (TextErrors[Boucle].Code != Code) ) { Boucle ++; }
			if (TextErrors[Boucle].Code == Code)
			{ MessageBox (TextErrors[Boucle].Message, MESSAGEBOX_OK); }
			else
			{
				char Message [128];
				sprintf (Message, "Error code %i", Code); 
				MessageBox (Message, MESSAGEBOX_OK);
			}
			break;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Fonctions privees                                                         //
///////////////////////////////////////////////////////////////////////////////


void C_Interface::Clean()
{
	int	Boucle = 0;
	
	for (Boucle = 0; Boucle < MAX_CHANNELS; Boucle++)
	{
		if (Channel[Boucle].Icon.Image != NULL) { MEM2_free(Channel[Boucle].Icon.Image); }
		Channel[Boucle].Icon.Image		= NULL;
		Channel[Boucle].Icon.Width		= 0;
		Channel[Boucle].Icon.Height		= 0;
		Channel[Boucle].Icon.NbImages	= 0;
		Channel[Boucle].IconId 			= 0;
		Channel[Boucle].FadeId			= 0;
		Channel[Boucle].Flags	 		= 0;
		Channel[Boucle].Name[0] = 0x00;
		memset (Channel[Boucle].Name, 0x00, C_Interface_MAX_NAME_SIZE);
	}
	
	RedrawCount			= 0;
	ChannelNbNavi		= 0;
	NbChannels			= 0;
	ChannelSelected_X	= 0;
	ChannelSelected_Y	= 0;
	
	this->CleanChannels();
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface::Read()
{
	if(Channel == NULL) { return; }
	
	Clean();
	this->ReadChannels();
	return;
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface::DrawBackground()
{
	int X, Y;

	for (Y = 0; Y < C_GFX::Height; Y += ImageRessource[ImageRessource_Background].Height)
	{
		for (X = 0; X < C_GFX::Width; X += ImageRessource[ImageRessource_Background].Width)
		{
			C_GFX::ShowImage (ImageRessource[ImageRessource_Background], X, Y, 0);
		}
	}
	
	C_GFX::ShowImage (ImageRessource[ImageRessource_Bar], 0, C_GFX::Height-(1+ImageRessource[ImageRessource_Bar].Height), 0);
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface::DrawChannelFade(int ChannelId)
{
	u32 NewIconId;
	int X, Y;

	if(Channel == NULL) { return; }
	X = ChannelId % CHANNELS_BY_LINE;
	Y = ChannelId / CHANNELS_BY_LINE;
	if ( Channel[ChannelId].IconId < (Channel[ChannelId].Icon.NbImages-1) )
	{ NewIconId = Channel[ChannelId].IconId+1; } else {  NewIconId = 0; }

	C_GFX::FusionImages (&TempImage, Channel[ChannelId].Icon, Channel[ChannelId].IconId, Channel[ChannelId].Icon, NewIconId,  Channel[ChannelId].FadeId);
	C_GFX::SetImageAlphaMask (TempImage, ImageRessource[ImageRessource_ThumbSprite]);
	C_GFX::ShowImage (TempImage, X*Channels_SpaceX+Channels_PaddingX , Y*Channels_SpaceY+Channels_PaddingY, 0);
	
	if (Channel[ChannelId].FadeId >= 248)
	{
		Channel[ChannelId].FadeId = 0;
		INTERFACE_FLAG_UnsetFlag (Channel[ChannelId].Flags, INTERFACE_FLAG_USEFADE);
		Channel[ChannelId].IconId = NewIconId;
		RedrawCount = 0;
	}
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface::DrawChannels (bool Select, bool ShowTitle)
{
	int X, Y;
	int ChannelId = MAX_CHANNELS - 1;
	u32 X0, Y0, DiffX1, DiffY1, DiffX2, DiffY2;

	if (Menu != NULL) { return; }
    if (HaveRedraw == REDRAW_ALL) { DrawBackground(); }
	Channels_PaddingYBottom  = ImageRessource[ImageRessource_Bar].Height + 10;

	X0 = ImageRessource[ImageRessource_ThumbDefault].Width;
	Y0 = ImageRessource[ImageRessource_ThumbDefault].Height;
	Channels_SpaceX = (C_GFX::Width - ((Channels_PaddingX*2) + (ImageRessource[ImageRessource_ThumbDefault].Width * CHANNELS_BY_LINE))) / (CHANNELS_BY_LINE - 1);
	Channels_SpaceX += ImageRessource[ImageRessource_ThumbDefault].Width;
	Channels_SpaceY = ((C_GFX::Height - Channels_PaddingYBottom) - ((Channels_PaddingY) + (ImageRessource[ImageRessource_ThumbDefault].Height * CHANNELS_BY_COL))) / (CHANNELS_BY_COL - 1);
	Channels_SpaceY += ImageRessource[ImageRessource_ThumbDefault].Height;

	DiffX1 = (ImageRessource[ImageRessource_ThumbUsed].Width/2)  - (X0/2);
	DiffY1 = (ImageRessource[ImageRessource_ThumbUsed].Height/2) - (Y0/2);

	DiffX2 = (ImageRessource[ImageRessource_ThumbEmpty].Width/2)  - (X0/2);
	DiffY2 = (ImageRessource[ImageRessource_ThumbEmpty].Height/2) - (Y0/2);

	for (Y = (CHANNELS_BY_COL - 1); Y > -1; Y--)
	{
		for (X = (CHANNELS_BY_LINE - 1); X > -1; X--)
		{
			if (Channel[ChannelId].Name[0] == 0x00)
			{
				C_GFX::ShowImage (ImageRessource[ImageRessource_ThumbEmpty], (X*Channels_SpaceX+(Channels_PaddingX)) - DiffX2 , (Y*Channels_SpaceY+(Channels_PaddingY)) - DiffY2, 0);
				
			}
			else
			{
				if (Channel[ChannelId].Icon.Image == NULL)
				{ 
					C_GFX::ShowImage (ImageRessource[ImageRessource_ThumbDefault], X*Channels_SpaceX+Channels_PaddingX , Y*Channels_SpaceY+Channels_PaddingY, 0);
					DrawChannelTxt(X, Y, ChannelId);
				}
				else
				{ 
					if (Channel[ChannelId].Icon.NbImages < 2)
					{
						C_GFX::ShowImage (Channel[ChannelId].Icon, X*Channels_SpaceX+Channels_PaddingX , Y*Channels_SpaceY+Channels_PaddingY, Channel[ChannelId].IconId);
					}
					else
					{
						if (Channel[ChannelId].Flags & INTERFACE_FLAG_USEFADE) 
						{ 
							Channel[ChannelId].FadeId += 4;
							DrawChannelFade(ChannelId);
						}
						else
						{
							C_GFX::ShowImage (Channel[ChannelId].Icon, X*Channels_SpaceX+Channels_PaddingX , Y*Channels_SpaceY+Channels_PaddingY, Channel[ChannelId].IconId);
						}
					}
					
					if (Channel[ChannelId].Flags & INTERFACE_FLAG_SHOWNAME) { DrawChannelTxt(X, Y, ChannelId); }
				}
				
				if ( (ChannelId != ChannelSelected_X + (ChannelSelected_Y * CHANNELS_BY_LINE)) || (Select == false) )
				{
					C_GFX::ShowImage (ImageRessource[ImageRessource_ThumbUsed], (X*Channels_SpaceX+Channels_PaddingX)-DiffX1 , (Y*Channels_SpaceY+Channels_PaddingY)-DiffY1, 0);
				}
			}
			
			ChannelId--;
		}
	}

	if (NbChannels > 0)
	{
		
		ChannelId = ChannelSelected_X + (ChannelSelected_Y * CHANNELS_BY_LINE);
		if (Select == true)
		{
			DiffX1 = (ImageRessource[ImageRessource_ThumbSelect].Width/2)  - (X0/2);
			DiffY1 = (ImageRessource[ImageRessource_ThumbSelect].Height/2) - (Y0/2);
			C_GFX::ShowImage (ImageRessource[ImageRessource_ThumbSelect], (ChannelSelected_X*Channels_SpaceX+Channels_PaddingX)-DiffX1, (ChannelSelected_Y*Channels_SpaceY+Channels_PaddingY)-DiffY1, 0);
		}
		
		if ( (Channel[ChannelId].Name[0] != 0x00) && (ShowTitle == true) )
		{
			X = (C_GFX::Width / 2) - (C_GFX::SizeOfString (Channel[ChannelId].Name) / 2);
			Y =  C_GFX::Height - (ImageRessource[ImageRessource_Bar].Height / 2);
			Y -= ImageRessource[ImageRessource_Font].Height / 2;
			C_GFX::ShowString (Channel[ChannelId].Name, X, Y, ColorTextBar );
		}
	}
	else if ((ShowTitle == true) && (EmptyMsg != NULL))
	{
		X = (C_GFX::Width / 2) - (C_GFX::SizeOfString (textRessource[TextRessource_NoHomebrewFound]) / 2);
		Y =  C_GFX::Height - (ImageRessource[ImageRessource_Bar].Height / 2);
		Y -= ImageRessource[ImageRessource_Font].Height / 2;
		C_GFX::ShowString (EmptyMsg, X, Y, ColorTextBar);
	}

	if (ChannelBlocId > 0)
	{
		C_GFX::ShowImage (ImageRessource[ImageRessource_Left], Channels_PaddingX - ImageRessource[ImageRessource_Left].Width, Channels_SpaceY+Channels_PaddingY-(ImageRessource[ImageRessource_Left].Height/2 - Y0/2), 0);
	}

	if ((ChannelNbNavi&2) == 2)
	{
		C_GFX::ShowImage (ImageRessource[ImageRessource_Right], C_GFX::Width - Channels_PaddingX - 1, Channels_SpaceY+Channels_PaddingY-(ImageRessource[ImageRessource_Right].Height/2 - Y0/2), 0);
	}
}

/////////////////////////////////////////////////////////////////////////////// 

void C_Interface::DrawChannelTxt(int X, int Y, int ChannelId)
{
	int Size, Boucle=0, Txt_X2 = 7, Txt_Y2 = 4;
	int Txt_X1 = ((X * Channels_SpaceX) + Channels_PaddingX + 7);
	int Txt_Y1 =  (Y * Channels_SpaceY) + Channels_PaddingY + 4;
	
	while (Channel[ChannelId].Name[Boucle] != 0)
	{
		if (Channel[ChannelId].Name[Boucle] == ' ')
		{
			if (Txt_X2 > 7)
			{
				Size = ImageRessource[ImageRessource_FontLight].Width / 4;
			}
			else { Size = 0; }
		}
		else
		{
			Size = C_GFX::ShowCaractere (ImageRessource[ImageRessource_FontLight], Channel[ChannelId].Name[Boucle], Txt_X1, Txt_Y1, TextColor, true);
		}
		
		Txt_X1 += Size;
		Txt_X2 += Size;
		
		if (Txt_X2 + ImageRessource[ImageRessource_FontLight].Width > ImageRessource[ImageRessource_ThumbDefault].Width)
		{
			Txt_X1 = ((X * Channels_SpaceX) + Channels_PaddingX + 7);
			Txt_X2 =5;
			Txt_Y1 += ImageRessource[ImageRessource_FontLight].Height - 1;
			Txt_Y2 += ImageRessource[ImageRessource_FontLight].Height - 1;
			if (Txt_Y2 + ImageRessource[ImageRessource_FontLight].Height > ImageRessource[ImageRessource_ThumbDefault].Height) break;
		}
		
		Boucle++;
	}	
}

/////////////////////////////////////////////////////////////////////////////// 

int C_Interface::SetImageChannel (u32 ChannelId, T_Image * ImageSrc)
{
	u32 Size;

	if (ImageSrc == NULL)			{ return ERROR_NULL_VALUE; }
	if (ImageSrc->Image == NULL)	{ return ERROR_NULL_VALUE; }
	
	if ( (ImageSrc->Width <= 132) && (ImageSrc->Height < 95) )
	{
		Channel[NbChannels].Icon.Image	= (T_RGBA*) MEM2_malloc (sizeof(T_RGBA)*132*95);
		Channel[NbChannels].Icon.Width	= 132;
		Channel[NbChannels].Icon.Height	= 95;
		Channel[ChannelId].Icon.NbImages= 1;
		C_GFX::FusionImages (&Channel[ChannelId].Icon, 0, ImageRessource[ImageRessource_ThumbDefault], 0, *ImageSrc, 0); 
		C_GFX::FusionImages (&Channel[ChannelId].Icon, 0, Channel[ChannelId].Icon, 0, ImageRessource[ImageRessource_WiiBar], 0); 	
		C_GFX::SetImageAlphaMask (Channel[ChannelId].Icon, ImageRessource[ImageRessource_ThumbSprite]);						
		return NO_ERROR;
	}
	
	if (ImageSrc->Width != 132) { return ERROR_BAD_DATA; } 
	Size = sizeof(T_RGBA) * ImageSrc->NbImages * ImageSrc->Width * ImageSrc->Height;
	Channel[NbChannels].Icon.Image	= (T_RGBA*) MEM2_malloc (Size);
	if (Channel[NbChannels].Icon.Image == NULL) { return ERROR_OUT_OF_MEMORY; } 
	Channel[ChannelId].Icon.Width		= ImageSrc->Width;
	Channel[ChannelId].Icon.Height		= ImageSrc->Height;
	Channel[ChannelId].Icon.NbImages	= ImageSrc->NbImages;
	memcpy (Channel[ChannelId].Icon.Image, ImageSrc->Image, Size);
	
	if (C_GFX::SetNbrImagesByHeight(&(Channel[ChannelId].Icon), 95) != NO_ERROR)
	{
		MEM2_free(Channel[ChannelId].Icon.Image	);
		Channel[ChannelId].Icon.Width		= 0;
		Channel[ChannelId].Icon.Height		= 0;
		Channel[ChannelId].Icon.NbImages	= 0;
		Channel[ChannelId].Icon.Image		= NULL;
		return ERROR_BAD_DATA;
	}
	
	C_GFX::SetImageAlphaMask (Channel[ChannelId].Icon, ImageRessource[ImageRessource_ThumbSprite]);
	
	return NO_ERROR;
}

/////////////////////////////////////////////////////////////////////////////// 
