///////////////////////////////////////////////////////////////////////////////
// Class C_Interface                                                         //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#ifndef _CLASS_UOBIHLLEH_C_INTERFACE
#define _CLASS_UOBIHLLEH_C_INTERFACE

#include "T_error.h"

#include "Wii_GFX.hpp"
#include "control.hpp"
#include "menu.hpp"
#include "FS.hpp"
#include "image_BMP.hpp"
#include "image_PNG.hpp"

#define CHANNELS_BY_LINE	4	/* Nombre de chaines par ligne  */
#define CHANNELS_BY_COL		3	/* Nombre de chaines par colone */
#define MAX_CHANNELS		(CHANNELS_BY_LINE * CHANNELS_BY_COL)

#define C_Interface_MAX_NAME_SIZE	1024

#define INTERFACE_CODE_EXIT			0
#define INTERFACE_CODE_LOOP			1

#define INTERFACE_FLAG_USEFADE		1
#define INTERFACE_FLAG_SHOWNAME		2
#define INTERFACE_FLAG_UnsetFlag(Var,Value) Var &= (0xFF ^ Value)

#define INTERFACE_CONTROL_ALLOW_ALL			0xFF
#define INTERFACE_CONTROL_ALLOW_NONE		0x00
#define INTERFACE_CONTROL_ALLOW_REFRESH		1


#define MESSAGEBOX_OK		1
#define MESSAGEBOX_NOWAIT	2
#define MESSAGEBOX_HIDE		3

#ifndef EXITCODE_SCHUTDOWN
#	define EXITCODE_SCHUTDOWN	1
#endif

#ifndef EXITCODE_WIIMENU
#	define EXITCODE_WIIMENU		2
#endif

class C_Interface
{
	public:
		static T_RGB ColorTextBar;
		static T_RGB ColorTextChannel;
		static T_RGB ColorTextMessageBox;
		static u8 ExitCode;
		
		C_Interface ();
		virtual ~C_Interface(void);
		
		void Init	();
		void MessageBox			(const char *, u8 Type);
		void ErrorMessageBox	(const int Code);
		
		virtual void SetRedrawAll 	();
		virtual void SetNoRedraw	();
		virtual int  Show (C_Control::T_Control Control = V_NULL);
		virtual void PostRetraceCallback	(u32 Count = 0);

	protected:
		typedef struct T_Channel
		{
			char    Name     [C_Interface_MAX_NAME_SIZE];
			T_Image Icon;
			u32     IconId;
			int		FadeId;
			u8	    Flags;
			
		};
		
		static C_Control::T_Control V_NULL;
		
		T_Channel * 	Channel;
		u32		ChannelBlocId;
		u32		NbChannels;
		u32		ChannelNbNavi;
		u32		ChannelSelected;
		T_RGB	TextColor;
		char *  EmptyMsg;
		u8		ControlAllowed;
		
		void Clean ();
		void Read();
		void DrawBackground  ();
		void DrawChannels    (bool Select, bool ShowTitle = true);
		void DrawChannelFade (int ChannelId);
		void DrawChannelTxt  (int X, int Y, int ChannelId); 
		int  SetImageChannel (u32 ChannelId, T_Image * ImageSrc);
		
		virtual void CleanChannels ()		= 0;
		virtual void ReadChannels  ()		= 0;
		virtual int  Run (int CHannelId)	= 0;
	
	private:
		static C_Menu *	Menu;
		static u8 BlocRedraw;
		
		int ChannelSelected_X ;
		int ChannelSelected_Y;
		int Channels_PaddingX;
		int Channels_SpaceX;
		int Channels_PaddingY;
		int Channels_SpaceY;
		int Channels_PaddingYBottom;
		u8			HaveRedraw;
		u32			RedrawCount;
		T_Image		TempImage;
};

#endif

///////////////////////////////////////////////////////////////////////////////
