///////////////////////////////////////////////////////////////////////////////
// Class C_Interface_App                                                     //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <string.h>

#include "asm.h"
#include "processor.h"
#include "kmemory.h"
#include "Ressources.h"

#include "interface_app.hpp"
#include "app_elf.hpp"
#include "app_dol.hpp"

///////////////////////////////////////////////////////////////////////////////

C_Interface_App::C_Interface_App (C_FileSystem * MyFS, const char * MyDirectory, u8 MyDirRecur, u8 MyFlags)
{
	FS			 = MyFS;
	Interface	 = NULL;	
	Channel_Data = NULL;
	DirRecur	 = MyDirRecur;
	Flags		 = MyFlags;
	EmptyMsg	 = textRessource[TextRessource_NoHomebrewFound];
	Directory	 = (char*) MEM2_malloc (strlen(MyDirectory)+1);
	if (Directory == NULL) { return; }
	strcpy (Directory, MyDirectory);
	Channel_Data = (T_InternalChannel *) MEM2_malloc(sizeof(T_InternalChannel) * MAX_CHANNELS);
	CleanChannels();
}

///////////////////////////////////////////////////////////////////////////////

C_Interface_App::~C_Interface_App()
{
	if (Interface != NULL)		{ delete Interface; }
	if (Directory != NULL)		{ MEM2_free (Directory); }
	if (Channel_Data != NULL)	{ MEM2_free (Channel_Data); }
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface_App::CleanChannels()
{
	if (Channel_Data == NULL) { return; }
	
	for (int Boucle =0; Boucle < MAX_CHANNELS; Boucle++)
	{
		Channel_Data[Boucle].Filename[0] = 0;
		Channel_Data[Boucle].Type = 0;
	}
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface_App::PostRetraceCallback (u32 Count = 0)
{
	if (Interface == NULL)
	{
		C_Interface::PostRetraceCallback (Count);
	}
	else
	{
		Interface->PostRetraceCallback (Count);
	}
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface_App::ReadChannels()
{
	char 		Filename[C_Interface_App_MAX_FILENAME_SIZE];
	int			HaveDir = 0;
	u32			Boucle = 0;
	s32			RetVal = 0;
	u8			DataFormat;
	u32			FileSize;
	u8 *		Data;
	C_Dir * 	Dir;
	T_Image		TempImg;

	
	if (Interface != NULL) { return Interface->ReadChannels(); }
	
	C_GFX::Begin();
		this->DrawChannels(false, false);
	C_GFX::End();
	FS->Mount();
	
InitChannelsRead:
	Dir = FS->NewDir();
	if (Dir->open (Directory) != 0)
	{
		delete Dir;
		FS->Unmount();
		
		if (ChannelBlocId > 0) 
		{
			Clean();
			ChannelBlocId = 0;
			goto InitChannelsRead; 
		}
		
		return;
	}
	
	Boucle = 0;
	HaveDir = Dir->next();

	while (Boucle < ChannelBlocId)
	{
		if (HaveDir < 0)
		{
			delete Dir;
			FS->Unmount();
			
			if (ChannelBlocId > 0) 
			{
				Clean();
				ChannelBlocId = 0;
				goto InitChannelsRead;
			}
			
			return;
		}
		
		if (Dir->Name[0] != '.')
		{
			if ((DirRecur > 0) && ((Dir->Attrib & DIR_ATTRIB_DIRECTORY) == DIR_ATTRIB_DIRECTORY))
			{
				sprintf (Filename, "%s/%s", Directory, Dir->Name);
				if (SubdirExist(FS, Filename) ) { Boucle++; }
			} 
			else
			{
				sprintf (Filename, "%s/%s/BOOT.DOL", Directory, Dir->Name);
				if (FS->FileExist(Filename)) { Boucle++; }
				else if ((Flags & C_Interface_App_SHOW_ELF) != 0)
				{
					sprintf (Filename, "%s/%s/BOOT.ELF", Directory, Dir->Name);
					if (FS->FileExist(Filename)) { Boucle++; }
				}
			}
		}
		
		HaveDir = Dir->next();
		if (HaveDir != 0) 
		{ 
			delete Dir;
			FS->Unmount();
			return; 
		}
	}

	while (HaveDir == 0)
	{
		if (Dir->Name[0] != '.')
		{
			if (NbChannels >= MAX_CHANNELS)
			{ 
				ChannelNbNavi |= 2;
				break;
			}
			
			sprintf (Filename, "%s/%s/BOOT.DOL", Directory, Dir->Name);
			
			if (FS->FileExist(Filename))
			{
				Channel_Data[NbChannels].Type = C_Interface_App_ISAPP;
				if (strlen(Filename) < C_Interface_App_MAX_FILENAME_SIZE)
				{
					sprintf (Channel_Data[NbChannels].Filename, "%s/BOOT.DOL", Dir->Name); 
				}
			}
			else if ((Flags & C_Interface_App_SHOW_ELF) != 0)
			{
				sprintf (Filename, "%s/%s/BOOT.ELF", Directory, Dir->Name);
				if (FS->FileExist(Filename))
				{
					Channel_Data[NbChannels].Type = C_Interface_App_ISAPP;
					if (strlen(Filename) < C_Interface_App_MAX_FILENAME_SIZE)
					{
						sprintf (Channel_Data[NbChannels].Filename, "%s/BOOT.ELF", Dir->Name);
					}
				}
			}
			
			if ((Channel_Data[NbChannels].Type == 0) && (Dir->Attrib & DIR_ATTRIB_DIRECTORY) && (DirRecur > 0))
			{
				sprintf (Filename, "%s/%s", Directory, Dir->Name);
				if (SubdirExist(FS, Filename) ) 
				{
					Channel_Data[NbChannels].Type = C_Interface_App_ISFOLDER;
					if (strlen(Dir->Name) < C_Interface_App_MAX_FILENAME_SIZE)
					{
						sprintf (Channel_Data[NbChannels].Filename, "%s", Dir->Name);
					}
				}
			}
			
			if (Channel_Data[NbChannels].Type != 0)
			{		
				// Chargement du titre du Homebrew
				C_File * File = FS->NewFile();
				sprintf (Filename, "%s/%s/TITLE.TXT", Directory, Dir->Name);
				RetVal = 0;
				
			    if ((File->open (Filename, FILE_OPEN_READ)) == 0) 
				{
					RetVal = File->read(Channel[NbChannels].Name, C_Interface_MAX_NAME_SIZE-1);
					if (RetVal > 0) { Channel[NbChannels].Name[RetVal] = 0x0; }
					File->close();
				}
				
				if (RetVal <= 0)
				{
					str_convert (Channel[NbChannels].Name, DEFAULT_STRING_PAGE_CODE, Dir->Name, FS->GetPageCode());
				}
				delete File;
				
				// Chargement image du Homebrew
				Channel[NbChannels].Icon.Image = NULL;
				sprintf (Filename, "%s/%s/TITLE.PNG", Directory, Dir->Name);
				DataFormat = 0;
				Data = (u8*) FS->LoadFile (Filename, &FileSize);
				
				if (Data == NULL) 
				{ 
					sprintf (Filename, "%s/%s/TITLE.BMP", Directory, Dir->Name);
					Data = (u8*) FS->LoadFile (Filename);
					DataFormat = 1;
					
					if (Data == NULL)
					{
						sprintf (Filename, "%s/%s/ICON.PNG", Directory, Dir->Name);
						Data = (u8*) FS->LoadFile (Filename, &FileSize);
						DataFormat = 0;
					}
				}
				
				if ((Data == NULL) && (Channel_Data[NbChannels].Type == C_Interface_App_ISFOLDER))
				{
					int Size;
				 
					Size  = ImageRessource[ImageRessource_Directory].Width * ImageRessource[ImageRessource_Directory].Height;
					Size *= ImageRessource[ImageRessource_Directory].NbImages * sizeof(T_RGBA);
					Channel[NbChannels].Icon.Image = (T_RGBA*) MEM2_malloc (Size);
					
					if (Channel[NbChannels].Icon.Image != NULL)
					{
						Channel[NbChannels].Icon.Width    = ImageRessource[ImageRessource_Directory].Width;
						Channel[NbChannels].Icon.Height   = ImageRessource[ImageRessource_Directory].Height;
						Channel[NbChannels].Icon.NbImages = ImageRessource[ImageRessource_Directory].NbImages;
						C_GFX::FusionImages (&Channel[NbChannels].Icon, 0, ImageRessource[ImageRessource_ThumbDefault], 0, ImageRessource[ImageRessource_Directory], 0); 
						C_GFX::FusionImages (&Channel[NbChannels].Icon, 0, Channel[NbChannels].Icon, 0, ImageRessource[ImageRessource_WiiBar], 0); 	
						C_GFX::SetImageAlphaMask (Channel[NbChannels].Icon, ImageRessource[ImageRessource_ThumbSprite]);	
					}
					
					Channel[NbChannels].Flags |= INTERFACE_FLAG_SHOWNAME;
				}
				else if (Data != NULL)
				{
					if (DataFormat == 0) 
					{ RetVal = Image_PNG::Decode (&TempImg, Data, FileSize); }
					else 
					{ RetVal = Image_BMP::Decode (&TempImg, Data); }
					MEM2_free(Data);
					
					if (RetVal == 0) 
					{
						SetImageChannel(NbChannels, &TempImg);
						MEM2_free(TempImg.Image);
					}
				}
				
				NbChannels++;
			}
			
			C_GFX::Begin();
				this->DrawChannels (false, false);
			C_GFX::End();
		}
		
		HaveDir = Dir->next();
	}
	
	delete Dir;
	FS->Unmount();
}

///////////////////////////////////////////////////////////////////////////////

int C_Interface_App::SubdirExist (C_FileSystem * MyFS, const char * HBDirectory)
{
	C_Dir * Dir;
	
	if (MyFS == NULL) { return false; }
	Dir = MyFS->NewDir();
	if (Dir->open(HBDirectory) != NO_ERROR)
	{
		delete Dir;
		return false;
	}
	
	while (Dir->next() == NO_ERROR)
	{
		if ( (Dir->Attrib & DIR_ATTRIB_DIRECTORY) && (Dir->Name[0] != '.') )
		{
			delete Dir;
			return true;
		}
	};
	
	delete Dir;
	return false;
}

///////////////////////////////////////////////////////////////////////////////
