///////////////////////////////////////////////////////////////////////////////
// Class C_Interface_Main                                                    //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <string.h>
#include <math.h>

#include "ressources.h"
#include "kmemory.h"
#include "interface_main.hpp"

// #define SHOW_ALL_CHANNELS

///////////////////////////////////////////////////////////////////////////////
// File systems                                                              //
///////////////////////////////////////////////////////////////////////////////

#include "FS_fatlib.hpp"
#include "FS_ISO9660.hpp"
#include "FS_SMB.hpp"


#define FS_NONE			0
#define FS_FRONTSD		1
#define FS_GECKOA		2
#define FS_GECKOB		3
#define FS_USB			4
#define FS_ISO9660		5
#define FS_SMB			6
#define FS_LAST			7

static const char * FS_Name [FS_LAST] =
{
	"Front SD",
	"Gecko A",
	"Gecko B",
	"USB",
	"DVD",
	"SMB"
};
	
static const u8 * FS_Icon [FS_LAST] =
{
	pngFsFrontSD,
	pngFsGecko,
	pngFsGecko,
	pngFsUSB,
	pngFsDVD,
	pngFsSMB
};
	
static const int FS_Icon_Length [FS_LAST] =
{
	pngFsFrontSD_Length,
	pngFsGecko_Length,
	pngFsGecko_Length,
	pngFsUSB_Length,
	pngFsDVD_Length,
	pngFsSMB_Length
};

///////////////////////////////////////////////////////////////////////////////

C_FileSystem * C_Interface_Main::CreateFS (int FsId)
{	
	switch (FsId)
	{
		case FS_FRONTSD:
			return new C_FS_Fatlib (PI_INTERNAL_SD);
		
		case FS_GECKOA:
			return new C_FS_Fatlib (PI_SDGECKO_A);
		
		case FS_GECKOB:
			return new C_FS_Fatlib (PI_SDGECKO_B);
		
		case FS_USB:
			return new C_FS_Fatlib (PI_USBSTORAGE);
		
		case FS_ISO9660:
			return new C_FS_ISO9660 ();
		
		case FS_SMB:
			return new C_FS_SMB (SMB_Server, SMB_Share, SMB_User, SMB_Password);
		
		default:
			return NULL;
	}
}


///////////////////////////////////////////////////////////////////////////////
// Channels                                                                  //
///////////////////////////////////////////////////////////////////////////////

#include "loader_channel.hpp"
#include "loader_hb.hpp"
#include "loader_emulator.hpp"
#include "loader_hbgc.hpp"
#include "loader_theme.hpp"


#define INTERFACE_NONE			0
#define INTERFACE_CHANNEL		1
#define INTERFACE_HOMEBREW		2
#define INTERFACE_APPS			3
#define INTERFACE_EMULATOR		4
#define INTERFACE_HOMEBREW_GC	5
#define INTERFACE_THEMES		6
#define COUNT_INTERFACES		7


static char * * INTERFACE_Name [COUNT_INTERFACES] =
{
	&textRessource[TextRessource_WiiChannels],
	&textRessource[TextRessource_Homebrew],
	&textRessource[TextRessource_Applications],
	&textRessource[TextRessource_Emulator],
	&textRessource[TextRessource_HomebrewGC],	
	&textRessource[TextRessource_Themes]
};

static u8 * INTERFACE_Icon [COUNT_INTERFACES] =
{
	(u8*) pngTitleWiiChannel,
	(u8*) pngTitleHomebrew,
	(u8*) pngTitleApps,
	(u8*) pngTitleEmulator,
	(u8*) pngTitleHomebrewGC,
	(u8*) NULL
};

static const int INTERFACE_Icon_Length [COUNT_INTERFACES] =
{
	pngTitleWiiChannel_Length,
	pngTitleHomebrew_Length,
	pngTitleApps_Length,
	pngTitleEmulator_Length,
	pngTitleHomebrewGC_Length,
	0
};

///////////////////////////////////////////////////////////////////////////////

u8 C_Interface_Main::InterfaceExist (C_FileSystem * MyFS, int FsId, int InterfaceId)
{
	switch (InterfaceId)
	{	
		case INTERFACE_CHANNEL:
			if (FsId != FS_NONE) { return false; }
		#ifdef SHOW_ALL_CHANNELS
			return true;
		#else
			return C_Loader_Channel::Exist ();
		#endif
		
		case INTERFACE_HOMEBREW:
			if (MyFS == NULL) { return false; }
		#ifdef SHOW_ALL_CHANNELS
			return true;
		#else
			return C_Loader_HB::Exist (MyFS, "/HOMEBREW");
		#endif
		
		case INTERFACE_APPS:
			if (MyFS == NULL) { return false; }
		#ifdef SHOW_ALL_CHANNELS
			return true;
		#else
			return C_Loader_HB::Exist (MyFS, "/APPS");
		#endif
		
		case INTERFACE_EMULATOR:
			if (MyFS == NULL) { return false; }
		#ifdef SHOW_ALL_CHANNELS
			return true;
		#else
			return C_Loader_Emulator::Exist (MyFS);
		#endif
		
		case INTERFACE_HOMEBREW_GC:
			if (MyFS == NULL) { return false; }
		#ifdef SHOW_ALL_CHANNELS
			return true;
		#else
			return C_Loader_HBGC::Exist (MyFS, "/GameCube");
		#endif		
		
		case INTERFACE_THEMES:
		{
			if (FsId != FS_FRONTSD) { return false; }
		#ifdef SHOW_ALL_CHANNELS
			return true;
		#else
			return C_Loader_Theme::Exist (MyFS);
		#endif
		}
		
		default:
			return false;
	}
}

///////////////////////////////////////////////////////////////////////////////

C_Interface * C_Interface_Main::CreateInterface (C_FileSystem * * MyFS, int InterfaceId)
{
	switch (InterfaceId)
	{
		case INTERFACE_CHANNEL:
			return new C_Loader_Channel ();
		
		case INTERFACE_HOMEBREW:
			return new C_Loader_HB (*MyFS, "/HOMEBREW");
		
		case INTERFACE_APPS:
			return new C_Loader_HB (*MyFS, "/APPS");
		
		case INTERFACE_EMULATOR:
			return new C_Loader_Emulator (*MyFS);
		
		case INTERFACE_HOMEBREW_GC:
			return new C_Loader_HBGC (*MyFS, "/GameCube");			
		
		case INTERFACE_THEMES:
			return new C_Loader_Theme (*MyFS);
		
		default:
			return NULL;
	}
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//// Classe C_Interface_Main                                               ////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

lwp_t C_Interface_Main::ProcessId = 0;

C_Interface_Main::C_Interface_Main (void) 
{
	float Temp;
	
	Interface = NULL;
	
	InternalChannel = (T_InternalChannel*) MEM2_malloc(sizeof(T_InternalChannel) * MAX_CHANNELS);
	memset (InternalChannel, 0x00, sizeof(T_InternalChannel) * MAX_CHANNELS);
	
	Temp = FS_LAST + COUNT_INTERFACES;
	Temp /= MAX_CHANNELS;
	MaxBlocs = ceil (Temp);
	BlocBegin = (T_InternalChannel*) MEM2_malloc(sizeof(T_InternalChannel) * MaxBlocs);
	memset (BlocBegin, 0x00, sizeof(T_InternalChannel) * MaxBlocs);
}

///////////////////////////////////////////////////////////////////////////////

C_Interface_Main::~C_Interface_Main(void)
{
	if (FS != NULL)			{ delete FS; }
	if (Interface != NULL)	{ delete Interface; }
	MEM2_free (BlocBegin);
	MEM2_free (InternalChannel);
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface_Main::SetRedrawAll ()
{
	if (Interface == NULL)
	{ C_Interface::SetRedrawAll (); }
	else
	{ Interface->SetRedrawAll ();   }
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface_Main::PostRetraceCallback	(u32 Count)
{
	if (Interface == NULL)
	{
		C_Interface::PostRetraceCallback (Count);
	}
	else
	{
		Interface->PostRetraceCallback (Count);
	}
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface_Main::CleanChannels () 
{
	int Boucle;

	for (Boucle = 0; Boucle < MAX_CHANNELS; Boucle++)
	{
		InternalChannel[Boucle].FSid 		= FS_NONE;
		InternalChannel[Boucle].InterfaceId	= INTERFACE_NONE;
	}
}

///////////////////////////////////////////////////////////////////////////////

void C_Interface_Main::ReadChannels ()
{
	EmptyMsg = NULL;
	C_GFX::Begin();
	this->SetRedrawAll();
	this->DrawChannels (false, false);
	C_GFX::End();

	LWP_CreateThread (&ProcessId,  C_Interface_Main::Process_ReadChannels, this, NULL, 8192, 50);
}

///////////////////////////////////////////////////////////////////////////////

void * C_Interface_Main::Process_ReadChannels (void * arg)
{	
	int		BoucleFsId;
	int		FistInterfaceId;
	int		BlocBeginId;
	char *	TempName;
	T_Image TempImage;
	C_FileSystem * MyFS = NULL;


	C_Interface_Main * This	= (C_Interface_Main *) arg;
	This->ControlAllowed	= INTERFACE_CONTROL_ALLOW_NONE;
	This->NbChannels		= 0;
	BlocBeginId				= This->ChannelBlocId / MAX_CHANNELS;
	TempImage.Image			= NULL;
	
	if (This->ChannelBlocId == 0)
	{
		This->BlocBegin[BlocBeginId].FSid = FS_NONE;
		This->BlocBegin[BlocBeginId].InterfaceId = 1;	
	}
	FistInterfaceId			= This->BlocBegin[BlocBeginId].InterfaceId;
	
	TempName = (char *) MEM2_malloc (C_Interface_MAX_NAME_SIZE);
	if (TempName == NULL) 
	{
		This->ControlAllowed = INTERFACE_CONTROL_ALLOW_ALL;
		return NULL; 
	}

	for (BoucleFsId = This->BlocBegin[BlocBeginId].FSid; BoucleFsId <= FS_LAST; BoucleFsId++)
	{
		MyFS = This->CreateFS (BoucleFsId);
#ifndef SHOW_ALL_CHANNELS
		if ((MyFS == NULL) && (BoucleFsId != FS_NONE) && (BoucleFsId != FS_LAST)) { continue; }
		
		if (MyFS != NULL)
		{
			if (MyFS->Mount() != NO_ERROR)
			{
				delete MyFS;
				continue;
			}
#endif			
			Image_PNG::Decode (&TempImage, FS_Icon[BoucleFsId-1], FS_Icon_Length[BoucleFsId-1]);
#ifndef SHOW_ALL_CHANNELS
		}
#endif
		
		for (int BoucleInterfaceId = FistInterfaceId; BoucleInterfaceId <= COUNT_INTERFACES; BoucleInterfaceId++)
		{
			if (InterfaceExist (MyFS, BoucleFsId, BoucleInterfaceId))
			{
				LWP_SetThreadPriority(This->ProcessId, LWP_PRIO_HIGHEST);
				if (This->NbChannels >= MAX_CHANNELS)
				{
					This->ChannelNbNavi |= 2;
					if (TempImage.Image != NULL) { MEM2_free(TempImage.Image); }
				#ifndef SHOW_ALL_CHANNELS
					if (MyFS != NULL) 
					{
						MyFS->Unmount(); 
						delete MyFS;
					}
				#endif
					if (BlocBeginId < This->MaxBlocs)
					{
						This->BlocBegin[BlocBeginId + 1].InterfaceId 	= BoucleInterfaceId;
						This->BlocBegin[BlocBeginId + 1].FSid			= BoucleFsId;
					}
					MEM2_free (TempName);
					This->SetRedrawAll();  					
					LWP_SetThreadPriority (This->ProcessId, 50);
					This->ControlAllowed = INTERFACE_CONTROL_ALLOW_ALL;
					
					return NULL;
				}
				else
				{
					memset (TempName, 0x00, C_Interface_MAX_NAME_SIZE);
					strcpy (TempName, *INTERFACE_Name[BoucleInterfaceId-1]);
					
					if (BoucleInterfaceId == INTERFACE_THEMES)
					{
						Image_PNG::Decode (&This->Channel[This->NbChannels].Icon,   pngTitleTheme,   pngTitleTheme_Length);	
						for (u32 Boucle = 0; Boucle < This->Channel[This->NbChannels].Icon.NbImages; Boucle++)
						{ C_GFX::FusionImages (&This->Channel[This->NbChannels].Icon, Boucle, This->Channel[This->NbChannels].Icon, Boucle, ImageRessource[ImageRessource_WiiBar], 0); }			
					}
					else
					{
						Image_PNG::Decode (&This->Channel[This->NbChannels].Icon,  INTERFACE_Icon[BoucleInterfaceId-1], INTERFACE_Icon_Length[BoucleInterfaceId-1]);
						
						if (MyFS != NULL) 
						{
							strcat (TempName, " (");
							strcat (TempName, FS_Name[BoucleFsId-1]);
							strcat (TempName, ")");
							C_GFX::FusionImages (&This->Channel[This->NbChannels].Icon, 0, This->Channel[This->NbChannels].Icon, 0, TempImage, 0); 
						}
						
						C_GFX::FusionImages (&This->Channel[This->NbChannels].Icon, 0, This->Channel[This->NbChannels].Icon, 0, ImageRessource[ImageRessource_WiiBar], 0); 
					}
					
					C_GFX::SetImageAlphaMask (This->Channel[This->NbChannels].Icon, ImageRessource[ImageRessource_ThumbSprite]);
					This->InternalChannel[This->NbChannels].FSid			= BoucleFsId;
					This->InternalChannel[This->NbChannels].InterfaceId	= BoucleInterfaceId;
					strcpy (This->Channel[This->NbChannels].Name, TempName);
					This->NbChannels++;
				}
				
				C_GFX::Begin();
				if (This->Interface == NULL) 
				{
					This->SetRedrawAll();
					This->DrawChannels (true, true);  
				}
				C_GFX::End();
				LWP_SetThreadPriority (This->ProcessId, 50);
			}
		}
		
		if (TempImage.Image != NULL) { MEM2_free(TempImage.Image); }
		
		if (MyFS != NULL) 
		{
			MyFS->Unmount();
			delete MyFS;
		}	
		
		FistInterfaceId = 1;
	}
	
	MEM2_free (TempName);
	This->ControlAllowed = INTERFACE_CONTROL_ALLOW_ALL;	
	return NULL;
}

///////////////////////////////////////////////////////////////////////////////

int C_Interface_Main::Run (int ChannelId)
{
	C_Control::T_Control Control;
	
	if ( (InternalChannel[ChannelId].FSid != FS_NONE) && (InternalChannel[ChannelId].FSid != FS_LAST) )
	{
		FS = CreateFS (InternalChannel[ChannelId].FSid);
		if (FS == NULL) { return ERROR_UNKNOW; }
	}
	else { FS = NULL; }
	
	Interface = CreateInterface (&FS, InternalChannel[ChannelId].InterfaceId);

	if (Interface != NULL)
	{
		SetNoRedraw();
		Interface->Init();
		Interface->SetRedrawAll();
		
		do
		{
			Control = C_Control::Read(0);		
			Interface->Show(Control);
			C_GFX::WaitVSync();
		} while ((Control.PAD & CONTROL_B) == 0);
		
		delete Interface;
		
		Interface = NULL;
		
		if (InternalChannel[ChannelSelected].InterfaceId == INTERFACE_THEMES)
		{
			this->Read();
		}
	}

	if (FS != NULL ) 
	{
		delete FS;
		FS = NULL; 
	}
	
	SetRedrawAll();
	C_Interface::Show ();
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////
