///////////////////////////////////////////////////////////////////////////////
// Class C_Loader_Emulator                                                   //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <string.h>

#include "kmemory.h"
#include "Ressources.h"

#include "loader_emulator.hpp"

const char C_Loader_Emulator::EmulatorDirectory [] = "/EMULATOR";

///////////////////////////////////////////////////////////////////////////////

int C_Loader_Emulator::Exist (C_FileSystem * MyFS)
{
	C_Dir * Dir;
	
	if (MyFS == NULL) { return false; }
	Dir = MyFS->NewDir();
	if (Dir->open(EmulatorDirectory) != NO_ERROR)
	{
		delete Dir;
		return false;
	}
	
	while (Dir->next() == NO_ERROR)
	{
		if ( (Dir->Attrib & DIR_ATTRIB_DIRECTORY) && (Dir->Name[0] != '.') )
		{
			delete Dir;
			return true;
		}
	};
	
	delete Dir;
	return false;
}

///////////////////////////////////////////////////////////////////////////////

C_Loader_Emulator::C_Loader_Emulator(C_FileSystem * MyFS): 
	C_Interface_App (MyFS, EmulatorDirectory, 0, C_Interface_App_SHOW_ELF)
{
	LoaderFiles = NULL;
}

///////////////////////////////////////////////////////////////////////////////

C_Loader_Emulator::~C_Loader_Emulator()
{
	if (LoaderFiles != NULL) { delete LoaderFiles; }
}

///////////////////////////////////////////////////////////////////////////////

void C_Loader_Emulator::PostRetraceCallback (u32 Count)
{
	if (LoaderFiles == NULL)
	{
		C_Interface_App::PostRetraceCallback (Count);
	}
	else
	{
		LoaderFiles->PostRetraceCallback (Count);
	}
}

///////////////////////////////////////////////////////////////////////////////

void C_Loader_Emulator::SetRedrawAll()
{
	if (LoaderFiles == NULL)
	{
		C_Interface_App::SetRedrawAll ();
	}
	else
	{
		LoaderFiles->SetRedrawAll ();
	}
}

///////////////////////////////////////////////////////////////////////////////

int C_Loader_Emulator::Run (int ChannelId)
{
	char * AppDir;
	char * RomDir;
	char * Ptr; 
	C_Control::T_Control Control;
	

	AppDir = (char *) MEM2_malloc(strlen(Channel_Data[ChannelId].Filename) + strlen(Directory) + 2);
	if (AppDir == NULL) { return ERROR_OUT_OF_MEMORY; }
	sprintf (AppDir, "%s/%s", Directory, Channel_Data[ChannelId].Filename);
	Ptr = AppDir + strlen(AppDir) - 1;
	while (Ptr[0] != 0x00)
	{
		if (Ptr[0] == '/') 
		{
			Ptr[0] = 0x00;
			break;
		}
		Ptr--;
	}
	
	RomDir = (char*) MEM2_malloc(strlen(AppDir+7));
	if (RomDir == NULL) 
	{
		MEM2_free(AppDir); 
		return ERROR_OUT_OF_MEMORY; 
	}
	
	sprintf (RomDir,"%s/ROMS", AppDir);
	LoaderFiles = new C_Loader_Files (FS, AppDir, RomDir);
	MEM2_free(AppDir);
	MEM2_free(RomDir);
	this->SetNoRedraw();
	
	LoaderFiles->Init();
	LoaderFiles->SetRedrawAll();
	
	do
	{
		Control = C_Control::Read(0);		
		LoaderFiles->Show (Control);
		C_GFX::WaitVSync();
	} while ((Control.PAD & CONTROL_B) == 0);
	
	delete LoaderFiles;
	LoaderFiles = NULL;
	this->SetRedrawAll();
	this->Show();	
	
	return NO_ERROR;	
}

///////////////////////////////////////////////////////////////////////////////
