///////////////////////////////////////////////////////////////////////////////
// Class C_Loader_Files                                                      //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <string.h>

#include "kmemory.h"
#include "Ressources.h"

#include "loader_files.hpp"
#include "app_elf.hpp"
#include "app_dol.hpp"
#include "system.h"


typedef void (*entrypoint) (void);

#define C_Loader_Files_ISROM		1
#define C_Loader_Files_ISFOLDER		2


///////////////////////////////////////////////////////////////////////////////

C_Loader_Files::C_Loader_Files(C_FileSystem * MyFS, const char * MyAppDirectory, const char * MyROMDirectory, u8 MyDirRecur)
{
	FS = MyFS;
	Interface = NULL;
	Channel_Data = NULL;
	DirRecur = MyDirRecur;
	EmptyMsg = textRessource[TextRessource_NoFilesFound];
	
	ROMDirectory = (char*) MEM2_malloc (strlen(MyROMDirectory)+1);
	if (ROMDirectory != NULL) { strcpy (ROMDirectory, MyROMDirectory); }
	
	AppDirectory = (char*) MEM2_malloc (strlen(MyAppDirectory)+1);
	if (ROMDirectory != NULL) { strcpy (AppDirectory, MyAppDirectory); }
	
	Channel_Data = (T_InternalChannel *) MEM2_malloc(sizeof(T_InternalChannel) * MAX_CHANNELS);
	CleanChannels();
}

///////////////////////////////////////////////////////////////////////////////

C_Loader_Files::~C_Loader_Files()
{
	if (Interface != NULL)		{ delete Interface; }
	if (ROMDirectory != NULL)	{ MEM2_free (ROMDirectory); }
	if (AppDirectory != NULL)	{ MEM2_free (AppDirectory); }
	if (Channel_Data != NULL)	{ MEM2_free (Channel_Data); }
}

///////////////////////////////////////////////////////////////////////////////

void C_Loader_Files::CleanChannels()
{
	if (Channel_Data == NULL) { return; }
	
	for (int Boucle = 0; Boucle < MAX_CHANNELS; Boucle++)
	{
		Channel_Data[Boucle].Filename[0] = 0;
		Channel_Data[Boucle].Type = 0;
	}
}

///////////////////////////////////////////////////////////////////////////////

void C_Loader_Files::PostRetraceCallback (u32 Count)
{
	if (Interface == NULL)
	{
		C_Interface::PostRetraceCallback (Count);
	}
	else
	{
		Interface->PostRetraceCallback (Count);
	}
}

///////////////////////////////////////////////////////////////////////////////

void C_Loader_Files::SetRedrawAll()
{
	if (Interface == NULL)
	{
		C_Interface::SetRedrawAll ();
	}
	else
	{
		Interface->SetRedrawAll ();
	}
}

///////////////////////////////////////////////////////////////////////////////

void C_Loader_Files::ReadChannels()
{
	char 	* Filename;
	int		HaveDir = 0;
	u32		Boucle = 0;
	s32		RetVal = 0;
	u8		DataFormat;
	u32		FileSize;
	u8 *	Data;
	C_Dir * Dir;

	C_GFX::Begin();
		this->DrawChannels(false, false);
	C_GFX::End();
	FS->Mount();
	Filename = (char*) MEM2_malloc (strlen (ROMDirectory) + C_Loader_Files_MAX_FILENAME_SIZE + 2);
	if (Filename == NULL) { return; }
	
InitChannelsRead:
	strcpy (Filename, ROMDirectory);
	FS->Mount();
	Dir = FS->NewDir();
	
	if (Dir->open (Filename) != 0)
	{
		delete Dir;
		FS->Unmount();
		
		if (ChannelBlocId > 0) 
		{
			Clean();
			ChannelBlocId = 0;
			goto InitChannelsRead; 
		}
		
		MEM2_free (Filename);
		return;
	}
	
	Boucle = 0;
	HaveDir = Dir->next();

	while (Boucle < ChannelBlocId)
	{
		if (HaveDir < 0)
		{
			delete Dir;
			FS->Unmount();
			
			if (ChannelBlocId > 0) 
			{
				Clean();
				ChannelBlocId = 0;
				goto InitChannelsRead;
			}
			
			MEM2_free (Filename);
			return;
		}
		
		RetVal = GetChannelType (Dir);
		if (RetVal > 0)	{ Boucle++;	}
		
		HaveDir = Dir->next();
		if (HaveDir != 0) 
		{ 
			delete Dir;
			FS->Unmount();
			MEM2_free (Filename);
			return; 
		}
	}

	while (HaveDir == 0)
	{
		RetVal = GetChannelType (Dir);
		
		if (RetVal > 0)
		{
			if (NbChannels >= MAX_CHANNELS)
			{ 
				ChannelNbNavi |= 2;
				break;
			}
			
			Channel_Data[NbChannels].Type = RetVal;
			
			// Titre
			str_convert (Channel[NbChannels].Name, DEFAULT_STRING_PAGE_CODE, Dir->Name, FS->GetPageCode());
			
			if (Channel_Data[NbChannels].Type == C_Loader_Files_ISROM)
			{
				for (int Boucle = strlen (Channel[NbChannels].Name); Boucle > 0; Boucle--)
				{
					if (Channel[NbChannels].Name[Boucle] == ' ') { break; }
					if (Channel[NbChannels].Name[Boucle] == '.') 
					{
						Channel[NbChannels].Name[Boucle] = 0x00;
						break; 
					}
				}
			}
			
			strncpy (Channel_Data[NbChannels].Filename, Dir->Name, C_Loader_Files_MAX_FILENAME_SIZE);
			Channel_Data[NbChannels].Filename[C_Loader_Files_MAX_FILENAME_SIZE] = 0x00;
			Channel[NbChannels].Icon.Image = NULL;
			if (Channel_Data[NbChannels].Type == C_Loader_Files_ISROM)
			{ sprintf (Filename, "%s/%s.PNG", ROMDirectory, Channel[NbChannels].Name); }
			else
			{ sprintf (Filename, "%s/%s/TITLE.PNG", ROMDirectory, Dir->Name); }
			DataFormat = 0;
			Data = (u8*) FS->LoadFile (Filename, &FileSize);
			
			if (Data == NULL) 
			{ 
				if (Channel_Data[NbChannels].Type == C_Loader_Files_ISROM)
				{ sprintf (Filename, "%s/%s.BMP", ROMDirectory, Channel[NbChannels].Name); }
				else
				{ sprintf (Filename, "%s/%s/TITLE.BMP", ROMDirectory, Dir->Name); }
				
				Data = (u8*) FS->LoadFile (Filename);
				DataFormat = 1;
			}
			
			if ((Data == NULL) && (Channel_Data[NbChannels].Type == C_Loader_Files_ISFOLDER))
			{
				int Size;
			 
				Size  = ImageRessource[ImageRessource_Directory].Width * ImageRessource[ImageRessource_Directory].Height;
				Size *= ImageRessource[ImageRessource_Directory].NbImages * sizeof(T_RGBA);
				Channel[NbChannels].Icon.Image = (T_RGBA*) MEM2_malloc (Size);
				
				if (Channel[NbChannels].Icon.Image != NULL)
				{
					Channel[NbChannels].Icon.Width    = ImageRessource[ImageRessource_Directory].Width;
					Channel[NbChannels].Icon.Height   = ImageRessource[ImageRessource_Directory].Height;
					Channel[NbChannels].Icon.NbImages = ImageRessource[ImageRessource_Directory].NbImages;
					C_GFX::FusionImages (&Channel[NbChannels].Icon, 0, ImageRessource[ImageRessource_ThumbDefault], 0, ImageRessource[ImageRessource_Directory], 0); 
					C_GFX::FusionImages (&Channel[NbChannels].Icon, 0, Channel[NbChannels].Icon, 0, ImageRessource[ImageRessource_WiiBar], 0); 	
					C_GFX::SetImageAlphaMask (Channel[NbChannels].Icon, ImageRessource[ImageRessource_ThumbSprite]);	
				}
				
				Channel[NbChannels].Flags |= INTERFACE_FLAG_SHOWNAME;
			}
			else if (Data != NULL)
			{
				T_Image TempImg;
				
				if (DataFormat == 0) 
				{ RetVal = Image_PNG::Decode (&TempImg, Data, FileSize); }
				else 
				{ RetVal = Image_BMP::Decode (&TempImg, Data); }
				
				if (RetVal == 0) 
				{
					SetImageChannel(NbChannels, &TempImg);
					MEM2_free (TempImg.Image); 
				}
				MEM2_free(Data);
			}
			
			NbChannels++;
			C_GFX::Begin();
				this->DrawChannels (false, false);
			C_GFX::End();
		}
		
		HaveDir = Dir->next();
	}

	delete Dir;
	MEM2_free (Filename);
	FS->Unmount();
}

///////////////////////////////////////////////////////////////////////////////

int C_Loader_Files::Run (int ChannelId)
{
	s32    Size;
	void * Data = NULL;
	char * FileLoader;
	char * CommandLine;
	const char * Drive;
	const char * DriveParam;
	C_App * App;
	C_Control::T_Control Control;
	
	if (Channel_Data[ChannelId].Filename[0] == 0x00) 
	{ 
		this->SetRedrawAll();
		this->Show();	
		return ERROR_BAD_DATA; 
	}
	
	// Si sous-repertoire
	if (Channel_Data[ChannelId].Type == C_Loader_Files_ISFOLDER)
	{
		char * NewDir;
		
		if (DirRecur == 0) { return NO_ERROR; }
		NewDir = (char *) MEM2_malloc (strlen (Channel_Data[ChannelId].Filename) + strlen(ROMDirectory) + 2);
		if (NewDir == NULL) { return ERROR_OUT_OF_MEMORY; } 
		sprintf (NewDir, "%s/%s", ROMDirectory, Channel_Data[ChannelId].Filename);
		this->SetNoRedraw();
		Interface = new C_Loader_Files (FS, AppDirectory, NewDir, DirRecur-1);
		MEM2_free(NewDir);
		Interface->Init();
		Interface->SetRedrawAll();
		
		do
		{
			Control = C_Control::Read(0);		
			Interface->Show (Control);
			C_GFX::WaitVSync();
		} while ((Control.PAD & CONTROL_B) == 0);
		
		delete Interface;
		Interface = NULL;
		this->SetRedrawAll();
		this->Show();		
		
		return NO_ERROR;
	}
	
	// Si fichier ROM...	
	Size = strlen(AppDirectory) + 10;
	FileLoader = (char*) MEM2_malloc(Size);
	
	if (FileLoader  == NULL) 
	{
		this->SetRedrawAll();
		this->Show();	
		return ERROR_OUT_OF_MEMORY;
	}

	Drive = FS->GetFSname();
	DriveParam = FS->GetStringParam();

	Size += (strlen (Drive) * 2) + 2;
	Size += strlen (ROMDirectory);
	Size += strlen (Channel_Data[ChannelId].Filename) + 2;
	if (DriveParam != NULL)	{ Size += 2 * strlen (DriveParam); }
	
	CommandLine = (char*) MEM2_malloc(Size);
	
	if (CommandLine == NULL) 
	{
		MEM2_free(FileLoader);
		this->SetRedrawAll();
		this->Show();	
		return ERROR_OUT_OF_MEMORY; 
	}

	sprintf (FileLoader, "%s/BOOT.DOL", AppDirectory);
	FS->Mount(); 
	Data = FS->LoadFile (FileLoader);

	if (Data == NULL)
	{
		sprintf (FileLoader, "%s/BOOT.ELF", AppDirectory);
		Data = FS->LoadFile (FileLoader);
		
		if (Data == NULL) 
		{
			FS->Unmount();
			MEM2_free(CommandLine);
			MEM2_free(FileLoader);
			this->SetRedrawAll();
			this->Show();	
			return FS_ERROR_CANT_LOAD_FILE;  
		}
	}
	
	if (DriveParam == NULL)
	{ sprintf (CommandLine, "%s:%s\t%s:%s/%s", Drive, FileLoader, Drive, ROMDirectory, Channel_Data[ChannelId].Filename);	}
	else { sprintf (CommandLine, "%s:%s%s\t%s:%s%s/%s", Drive, DriveParam, FileLoader, Drive, DriveParam, ROMDirectory, Channel_Data[ChannelId].Filename); }	
	MEM2_free(FileLoader);
	FS->Unmount();
	
	// On attend que l'utilisateur relache les boutons
	while ( C_Control::Read(0).PAD != 0) { C_GFX::WaitVSync(); }
	
	// Verification du programme, chargement du point d'entree
	if (C_App_elf::isElf(Data) == true) 
	{ App = new C_App_elf(); }
	else 
	{ App = new C_App_dol(); }
  
	App->Launch(Data, CommandLine);
	delete App;
	MEM2_free(Data);
	MEM2_free(CommandLine);
	this->SetRedrawAll();
	this->Show();	
	return -ENOEXEC; 		
}

///////////////////////////////////////////////////////////////////////////////

u8 C_Loader_Files::GetChannelType (C_Dir * Dir)
{
	char * Ptr;
	
	if (strlen (Dir->Name) > C_Loader_Files_MAX_FILENAME_SIZE) { return 0; }
 
	if (Dir->Attrib & DIR_ATTRIB_DIRECTORY) 
	{
		if (Dir->Name[0] == '.') { return 0; }
		if (DirRecur == 0) { return 0; }
		return C_Loader_Files_ISFOLDER;
	}
	else
	{
		
		if (stricmp (Dir->Name, "title.txt") == 0) { return 0; }
		Ptr = (char*) Dir->Name + strlen (Dir->Name);
		
		for (int Boucle = strlen (Dir->Name); Boucle > 0; Boucle--)
		{
			if (*Ptr == '.') 
			{
				if      (stricmp(strlwr (Ptr), ".bmp") == 0) { return 0; }
				else if (stricmp(strlwr (Ptr), ".png") == 0) { return 0; }
				break; 
			}
			Ptr--;
		}
		
	}
	
	return C_Loader_Files_ISROM;
}

///////////////////////////////////////////////////////////////////////////////
