///////////////////////////////////////////////////////////////////////////////
// Class C_Loader_HB                                                         //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include "loader_hb.hpp"
#include "app_elf.hpp"
#include "app_dol.hpp"
#include "kmemory.h"
#include "processor.h"
#include "system.h"

typedef void (*entrypoint) (void);

///////////////////////////////////////////////////////////////////////////////

int C_Loader_HB::Exist (C_FileSystem * MyFS, const char * HBDirectory)
{
	return SubdirExist (MyFS, HBDirectory);
}

///////////////////////////////////////////////////////////////////////////////

C_Loader_HB::C_Loader_HB(C_FileSystem * MyFS, const char * MyDirectory, u8 MyDirRecur):
	C_Interface_App (MyFS, MyDirectory, MyDirRecur, C_Interface_App_SHOW_ELF) 
{};

///////////////////////////////////////////////////////////////////////////////

int C_Loader_HB::Run (int ChannelId)
{
	u32    Size;
	void * Data = NULL;
	char * CmdLine;
	const char * Loader_FSname;
	const char * Loader_FSparam;
	C_App * App;
	C_Control::T_Control Control;
	

	if (Channel_Data[ChannelId].Filename[0] == 0x00) 
	{
		this->SetRedrawAll();
		this->Show();	
		return ERROR_BAD_DATA;
	}
	
	Loader_FSname	= FS->GetFSname();
	Loader_FSparam	= FS->GetStringParam();

	Size  = strlen (Loader_FSname) + 1;
	Size += strlen (Directory) + 1;
	Size += strlen (Channel_Data[ChannelId].Filename) + 2;
	if (Loader_FSparam != NULL) { Size += strlen (Loader_FSparam); }
	
	CmdLine = (char*) MEM2_malloc(Size);
	if (CmdLine  == NULL) 
	{
		this->SetRedrawAll();
		this->Show();	
		return ERROR_OUT_OF_MEMORY;
	}	

	sprintf (CmdLine, "%s/%s", Directory, Channel_Data[ChannelId].Filename);

	// Si sous-repertoire
	if (Channel_Data[ChannelId].Type == C_Interface_App_ISFOLDER)
	{
		if (DirRecur == 0) { return NO_ERROR; }
		Interface = new C_Loader_HB (FS, CmdLine, DirRecur-1);
		Interface->Init();
		Interface->SetRedrawAll();
		
		do
		{
			Control = C_Control::Read(0);		
			Interface->Show(Control);
			C_GFX::WaitVSync();
		} while ((Control.PAD & CONTROL_B) == 0);
		
		delete Interface;
		Interface = NULL;
		MEM2_free(CmdLine);
		this->SetRedrawAll();
		this->Show();	
		return NO_ERROR;
	}
	
	
	// Si fichier homebrew...
	FS->Mount();
	Data = FS->LoadFile (CmdLine);
	
	if (Data == NULL) 
	{
		MEM2_free(CmdLine);
		FS->Unmount();
		this->SetRedrawAll();
		this->Show();	
		return FS_ERROR_CANT_LOAD_FILE; 
	}

	if (Loader_FSparam == NULL)
	{ sprintf (CmdLine, "%s:%s/%s", Loader_FSname, Directory, Channel_Data[ChannelId].Filename); }
	else { sprintf (CmdLine, "%s:%s%s/%s", Loader_FSname, Loader_FSparam, Directory, Channel_Data[ChannelId].Filename); }
	FS->Unmount();
	
	// On attend que l'utilisateur relache les boutons
	while ( C_Control::Read(0).PAD != 0) { C_GFX::WaitVSync(); }
	
	// Verification du programme, chargement du point d'entree
	if (C_App_elf::isElf(Data) == true) { App = new C_App_elf(); } 
	else { App = new C_App_dol(); }	
  
	App->Launch(Data, CmdLine);
	delete App;
	MEM2_free(CmdLine);
	MEM2_free(Data);
	this->SetRedrawAll();
	this->Show();	
	return -ENOEXEC; 	
}

///////////////////////////////////////////////////////////////////////////////
