///////////////////////////////////////////////////////////////////////////////
// Class C_Loader_HBGC                                                       //
// Created by Hell Hibou (2009)                                              //
// based on sources of Wii GameCube Homebrew Launcher by MegaMan             //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include "../../common.h"
#include "../../AppConfig.h"

#include "loader_hbgc.hpp"
#include "app_elf.hpp"
#include "app_dol.hpp"
#include "kmemory.h"
#include "processor.h"
#include "system.h"
#include "ressources.h"
#include "WiiDVD.hpp"

extern void BeforeExit  ();

#define MAGIC_GAMECUBE_DISC	0xC2339F3DL

///////////////////////////////////////////////////////////////////////////////

int C_Loader_HBGC::Exist (C_FileSystem * MyFS, const char * HBDirectory)
{
	return SubdirExist (MyFS, HBDirectory);
}

///////////////////////////////////////////////////////////////////////////////

C_Loader_HBGC::C_Loader_HBGC(C_FileSystem * MyFS, const char * MyDirectory, u8 MyDirRecur):
	C_Interface_App (MyFS, MyDirectory, MyDirRecur, 0) 
{}

///////////////////////////////////////////////////////////////////////////////

int C_Loader_HBGC::Run (int ChannelId)
{
	u32    		Size;
	int			RetVal;
	u8 *		Data		= NULL;
	char *		Filename	= NULL;
	C_File * 	File		= NULL;
	C_Control::T_Control Control;
	static tikview View ATTRIBUTE_ALIGN(32);

	if (Channel_Data[ChannelId].Filename[0] == 0x00) 
	{
		this->SetRedrawAll();
		this->Show();	
		return ERROR_BAD_DATA;
	}

	Size  = strlen (Directory) + 1;
	Size += strlen (Channel_Data[ChannelId].Filename) + 2;
	
	Filename = (char*) MEM2_malloc(Size);
	if (Filename  == NULL) 
	{
		this->SetRedrawAll();
		this->Show();	
		return ERROR_OUT_OF_MEMORY;
	}	

	sprintf (Filename, "%s/%s", Directory, Channel_Data[ChannelId].Filename);

	// Si sous-repertoire
	if (Channel_Data[ChannelId].Type == C_Interface_App_ISFOLDER)
	{
		if (DirRecur == 0) { return NO_ERROR; }
		Interface = new C_Loader_HBGC (FS, Filename, DirRecur-1);
		Interface->Init();
		Interface->SetRedrawAll();
		
		do
		{
			Control = C_Control::Read(0);		
			Interface->Show(Control);
			C_GFX::WaitVSync();
		} while ((Control.PAD & CONTROL_B) == 0);
		
		delete Interface;
		Interface = NULL;
		MEM2_free(Filename);
		this->SetRedrawAll();
		this->Show();	
		return NO_ERROR;
	}
	
	
	// Si fichier homebrew...
	MessageBox (textRessource[TextRessource_HomebrewGC_Load], MESSAGEBOX_NOWAIT);
	
	FS->Mount();
	File =  FS->NewFile();
	Data = (u8*) 0x80800000;
	
	RetVal = File->open(Filename, FILE_OPEN_READ);
	if (RetVal != NO_ERROR) { goto L_Loader_HBGC_Run_End; }
	if (File->Size > (0x81330000 - 0x80800000)) 
	{
		RetVal = -ENOEXEC;
		goto L_Loader_HBGC_Run_End; 
	}
	
	Size = 0;
	do
	{
		RetVal = File->read(Data, 0x800);
		Size += RetVal;
		Data += RetVal;
	} while (RetVal > 0);
	
	DCFlushRange ((u8*) 0x80800000, Size);
	File->close();
	delete File;
	FS->Unmount();


	
	RetVal = ES_GetTicketViews(0x0000000100000100ULL, &View, 1);
	if (RetVal != NO_ERROR) { goto L_Loader_HBGC_Run_End; }
	
	// Read GameID from DVD
	if (IOS_GetVersion() > 10) { SetLED (2,1); }
	RetVal = C_WiiDVD::Init();
	if (RetVal == NO_ERROR)
	{
		C_WiiDVD::Reset();
		memset ((void*)0x80000000, 0, 0x20);
		RetVal = C_WiiDVD::ReadID ((void*)0x80000000);
		if (RetVal >= 0)
		{
			if (*((u32*)0x8000001C) != MAGIC_GAMECUBE_DISC) 
			{ RetVal = ERROR_BAD_DATA; }
		}
		C_WiiDVD::Deinit();
	}
	if (IOS_GetVersion() > 10) { SetLED (2,0); } 

	// Make fake GameID
	if (RetVal < 0)
	{
		char * GameId;
		
		GameId = (char*)0x80000000;
		memset (GameId, 0, 0x20);
		memcpy (GameId, "GHL?01", 6);
		*((u32*)0x8000001C) = MAGIC_GAMECUBE_DISC;
		
		switch (CONF_GetRegion())
		{
			case CONF_REGION_JP:
				GameId[3] = 'J';
				break;
			
			case CONF_REGION_US:
				GameId[3] = 'E';
				break;	
			
			case CONF_REGION_EU:
				GameId[3] = 'P';
				break;	
				
			case CONF_REGION_KR:
				GameId[3] = 'J';
				break;	
			
			default:
				GameId[3] = 'E';
				break;
		}
		
	}
	
	DCFlushRange((unsigned char *) 0x80000000, 0x20);
	strcpy ((char *) 0x807FFFE0, "gchomebrew dol");
	DCFlushRange((unsigned char *) 0x807FFFE0, 32);
	
	BeforeExit();
	*(volatile unsigned int *)0xCC003024 |= 7;
	ES_LaunchTitle(0x0000000100000100ULL, &View);
	Reload();

L_Loader_HBGC_Run_End:
	MessageBox (NULL, MESSAGEBOX_HIDE);
	if (Filename != NULL) { MEM2_free(Filename); }
	this->SetRedrawAll();
	this->Show();		
	return RetVal; 	
}

///////////////////////////////////////////////////////////////////////////////
