///////////////////////////////////////////////////////////////////////////////
// Class C_Control                                                           //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <ogc/pad.h>
#include "wii_gfx.hpp"
#include "kmemory.h"
#include "control.hpp"
#include "AppConfig.h"


ir_t  C_Control::Wiimote_IR;
u8    C_Control::OldValid [4] = { 0, 0, 0, 0 };

///////////////////////////////////////////////////////////////////////////////

void C_Control::Init ()
{
	PAD_Init();
	WPAD_Init();
	
#ifdef ENABLE_IR_CURSOR
	WPAD_SetVRes(WPAD_CHAN_0,C_GFX::Width,C_GFX::Height);
	WPAD_SetDataFormat(WPAD_CHAN_0,WPAD_FMT_BTNS_ACC_IR);  
#endif
    
}

///////////////////////////////////////////////////////////////////////////////

void C_Control::Stop ()
{
	WPAD_Shutdown();
}

///////////////////////////////////////////////////////////////////////////////

C_Control::T_Control C_Control::Read (int Id)
{
	C_Control::T_Control RetVal = { 0, false, 0, 0 };
	struct expansion_t ext;
	int GCpad;
	u32 WiiPad;
	int X, Y;
   
  
 	// Mannette GC
	PAD_ScanPads();
	GCpad  = PAD_ButtonsDown(Id);
	if ((GCpad & PAD_BUTTON_UP)		!= 0) { RetVal.PAD |= CONTROL_UP;			}         
	if ((GCpad & PAD_BUTTON_DOWN)	!= 0) { RetVal.PAD |= CONTROL_DOWN;		}       
	if ((GCpad & PAD_BUTTON_LEFT)	!= 0) { RetVal.PAD |= CONTROL_LEFT;		}       
	if ((GCpad & PAD_BUTTON_RIGHT)	!= 0) { RetVal.PAD |= CONTROL_RIGHT;		}    
	if ((GCpad & PAD_BUTTON_A)		!= 0) { RetVal.PAD |= CONTROL_A;			}       
	if ((GCpad & PAD_BUTTON_B)		!= 0) { RetVal.PAD |= CONTROL_B;			}
	if ((GCpad & PAD_TRIGGER_L)		!= 0) { RetVal.PAD |= CONTROL_L_MINUS;		}    
	if ((GCpad & PAD_TRIGGER_R)		!= 0) { RetVal.PAD |= CONTROL_R_PLUS;		}      
	if ((GCpad & PAD_BUTTON_START)	!= 0) { RetVal.PAD |= CONTROL_START_HOME;	} 
	if ((GCpad & PAD_BUTTON_X)		!= 0) { RetVal.PAD |= CONTROL_X;			}  
	if ((GCpad & PAD_BUTTON_Y)		!= 0) { RetVal.PAD |= CONTROL_Y;			}  
	if ((GCpad & PAD_TRIGGER_Z)		!= 0) { RetVal.PAD |= CONTROL_Z_1;			} 
	
	X = PAD_StickX(Id);
	Y = -PAD_StickY(Id);
	if (Y < -50)	{ RetVal.PAD |= CONTROL_UP;   }
	if (Y >  50)	{ RetVal.PAD |= CONTROL_DOWN; } 
	if (X < -50)	{ RetVal.PAD |= CONTROL_LEFT; }
	if (X >  50)	{ RetVal.PAD |= CONTROL_RIGHT;}
  
   
   
	// Wiimote
	WPAD_ScanPads();
	WiiPad =  WPAD_ButtonsDown(Id);
	if ((WiiPad & WPAD_BUTTON_UP)		!= 0) { RetVal.PAD |= CONTROL_UP;			}         
	if ((WiiPad & WPAD_BUTTON_DOWN)		!= 0) { RetVal.PAD |= CONTROL_DOWN;		}       
	if ((WiiPad & WPAD_BUTTON_LEFT)		!= 0) { RetVal.PAD |= CONTROL_LEFT;		}       
	if ((WiiPad & WPAD_BUTTON_RIGHT)	!= 0) { RetVal.PAD |= CONTROL_RIGHT;		}    
	if ((WiiPad & WPAD_BUTTON_A)		!= 0) { RetVal.PAD |= CONTROL_A;			}       
	if ((WiiPad & WPAD_BUTTON_B)		!= 0) { RetVal.PAD |= CONTROL_B;			}
	if ((WiiPad & WPAD_BUTTON_MINUS)	!= 0) { RetVal.PAD |= CONTROL_L_MINUS;		}    
	if ((WiiPad & WPAD_BUTTON_PLUS)		!= 0) { RetVal.PAD |= CONTROL_R_PLUS;		}     
	if ((WiiPad & WPAD_BUTTON_HOME)		!= 0) { RetVal.PAD |= CONTROL_START_HOME;	}
	if ((WiiPad & WPAD_BUTTON_1)		!= 0) { RetVal.PAD |= CONTROL_Z_1;			}
   
   
   
	// Manette Classique
	WPAD_Expansion(WPAD_CHAN_0, &ext);
	if (ext.type == WPAD_EXP_CLASSIC)
	{
		if ((WiiPad & WPAD_CLASSIC_BUTTON_UP)		!= 0) { RetVal.PAD |= CONTROL_UP;			}         
		if ((WiiPad & WPAD_CLASSIC_BUTTON_DOWN)		!= 0) { RetVal.PAD |= CONTROL_DOWN;		}       
		if ((WiiPad & WPAD_CLASSIC_BUTTON_LEFT)		!= 0) { RetVal.PAD |= CONTROL_LEFT;		}       
		if ((WiiPad & WPAD_CLASSIC_BUTTON_RIGHT)	!= 0) { RetVal.PAD |= CONTROL_RIGHT;		}    
		if ((WiiPad & WPAD_CLASSIC_BUTTON_A)		!= 0) { RetVal.PAD |= CONTROL_A;			}       
		if ((WiiPad & WPAD_CLASSIC_BUTTON_B)		!= 0) { RetVal.PAD |= CONTROL_B;			}
		if ((WiiPad & WPAD_CLASSIC_BUTTON_MINUS)	!= 0) { RetVal.PAD |= CONTROL_L_MINUS;		}    
		if ((WiiPad & WPAD_CLASSIC_BUTTON_PLUS)		!= 0) { RetVal.PAD |= CONTROL_R_PLUS;		}
		if ((WiiPad & WPAD_CLASSIC_BUTTON_FULL_L)	!= 0) { RetVal.PAD |= CONTROL_L_MINUS;		}    
		if ((WiiPad & WPAD_CLASSIC_BUTTON_FULL_R)	!= 0) { RetVal.PAD |= CONTROL_R_PLUS;		}      
		if ((WiiPad & WPAD_CLASSIC_BUTTON_HOME)		!= 0) { RetVal.PAD |= CONTROL_START_HOME;	} 
		if ((WiiPad & WPAD_CLASSIC_BUTTON_X)		!= 0) { RetVal.PAD |= CONTROL_X;			}  
		if ((WiiPad & WPAD_CLASSIC_BUTTON_Y)		!= 0) { RetVal.PAD |= CONTROL_Y;			}  
		if ((WiiPad & WPAD_CLASSIC_BUTTON_ZL)		!= 0) { RetVal.PAD |= CONTROL_Z_1;			}  
		if ((WiiPad & WPAD_CLASSIC_BUTTON_ZR)		!= 0) { RetVal.PAD |= CONTROL_Z_1;			} 
		
		X =   ((ext.classic.ljs.pos.x - ext.classic.ljs.center.x) * 200 / (ext.classic.ljs.max.x - ext.classic.ljs.min.x));
		Y = - ((ext.classic.ljs.pos.y - ext.classic.ljs.center.y) * 200 / (ext.classic.ljs.max.y - ext.classic.ljs.min.y));
		if (Y < -50)	{ RetVal.PAD |= CONTROL_UP;   }
		if (Y >  50)	{ RetVal.PAD |= CONTROL_DOWN; } 
		if (X < -50)	{ RetVal.PAD |= CONTROL_LEFT; }
		if (X >  50)	{ RetVal.PAD |= CONTROL_RIGHT;}
	}

	// Pointeur Infra-rouge
#ifdef ENABLE_IR_CURSOR
	OldValid[Id] = Wiimote_IR.valid * 2;
	WPAD_IR(Id, &Wiimote_IR);
	if (Wiimote_IR.valid) { RetVal.IrValid = CONTROL_VALID | OldValid[Id]; }
	else { RetVal.IrValid =  OldValid[Id]; }
	RetVal.IrX 		= Wiimote_IR.x;
	RetVal.IrY		= Wiimote_IR.y;
#else
	RetVal.IrValid	= false;
	RetVal.IrX		= 0;
	RetVal.IrY		= 0;
#endif

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////

void C_Control::ShutdownWiimote ()
{
	s32 Fd = -1;
	
	char bluetoothResetData1[] __attribute__((aligned(32))) = { 0x20 };             /* bmRequestType */
	char bluetoothResetData2[] __attribute__((aligned(32))) = { 0x00 };             /* bmRequest */
	char bluetoothResetData3[] __attribute__((aligned(32))) = { 0x00, 0x00 };       /* wValue */
	char bluetoothResetData4[] __attribute__((aligned(32))) = { 0x00, 0x00 };       /* wIndex */
	char bluetoothResetData5[] __attribute__((aligned(32))) = { 0x03, 0x00 };       /* wLength */
	char bluetoothResetData6[] __attribute__((aligned(32))) = { 0x00 };             /* unknown; set to zero. */
	char bluetoothResetData7[] __attribute__((aligned(32))) = { 0x03, 0x0c, 0x00 }; /* Mesage payload. */

	ioctlv bluetoothReset[] __attribute__((aligned(32))) =   /** Vectors of data transfered. */
	{
		{ bluetoothResetData1, sizeof(bluetoothResetData1) },
		{ bluetoothResetData2, sizeof(bluetoothResetData2) },
		{ bluetoothResetData3, sizeof(bluetoothResetData3) },
		{ bluetoothResetData4, sizeof(bluetoothResetData4) },
		{ bluetoothResetData5, sizeof(bluetoothResetData5) },
		{ bluetoothResetData6, sizeof(bluetoothResetData6) },
		{ bluetoothResetData7, sizeof(bluetoothResetData7) }
	};

	Fd = IOS_Open("/dev/usb/oh1/57e/305", 2);
	IOS_Ioctlv(Fd, 0, 6, 1, bluetoothReset);
	IOS_Close(Fd);
}

///////////////////////////////////////////////////////////////////////////////

void C_Control::SetPowerCallback (WPADShutdownCallback cb)
{
	WPAD_SetPowerButtonCallback (cb);
}

///////////////////////////////////////////////////////////////////////////////
