///////////////////////////////////////////////////////////////////////////////
// Class C_GFX                                                               //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <gccore.h>
#include <string.h>
#include <stdio.h>
#include "Ressources.h"
#include "kmemory.h"
#include "T_error.h"

#include "Wii_GFX.hpp"

#define FLUSH_OFF		0
#define FLUSH_VRAM		1
#define FLUSH_VIDEORAM	2
#define FLUSH_FULL		3

///////////////////////////////////////////////////////////////////////////////

T_RGB *			C_GFX::VRAM			= NULL;
u32 *			C_GFX::VideoRAM[2]	= { NULL, NULL };
GXRModeObj *	C_GFX::vmode		= NULL;
u8 				C_GFX::VRAM_Page	= 0;
u8    			C_GFX::HaveFlush	= FLUSH_OFF;
s32				C_GFX::Height		= 0;
s32				C_GFX::Width		= 0;


///////////////////////////////////////////////////////////////////////////////

void C_GFX::Init()
{
	VIDEO_Init  ();
	vmode = VIDEO_GetPreferredMode(NULL);
	VIDEO_Configure (vmode);
	
	VideoRAM[0] = (u32 *) MEM_K0_TO_K1 (SYS_AllocateFramebuffer (vmode));
	VideoRAM[1] = (u32 *) MEM_K0_TO_K1 (SYS_AllocateFramebuffer (vmode));
	Height		= vmode->xfbHeight;
	Width		= vmode->fbWidth;
	VRAM_Page 	= 0;
	VRAM 		= (T_RGB *) MEM2_malloc(vmode->fbWidth * vmode->xfbHeight * sizeof(T_RGBA));

	console_init(VideoRAM[0], 20 ,20 ,vmode->fbWidth - 20, vmode->xfbHeight - 20, vmode->fbWidth * VI_DISPLAY_PIX_SZ);
	VIDEO_ClearFrameBuffer (vmode, VideoRAM[0], COLOR_BLACK);
	VIDEO_ClearFrameBuffer (vmode, VideoRAM[1], COLOR_BLACK);
	Clear (RGB_BLACK);
	VIDEO_SetNextFramebuffer (VideoRAM[0]);
	VIDEO_SetBlack (false);	
	VIDEO_Flush ();
	VIDEO_WaitVSync ();
	if (vmode->viTVMode & VI_NON_INTERLACE) { VIDEO_WaitVSync (); }
	
	HaveFlush = FLUSH_VIDEORAM;
	VRAM_Page = 1;
}

///////////////////////////////////////////////////////////////////////////////

void C_GFX::Begin ()
{
	VRAM_Page ^= 1;
	HaveFlush = FLUSH_OFF;
};

///////////////////////////////////////////////////////////////////////////////

void C_GFX::Flush()
{
	int y1, cb1, cr1, y2, cb2, cr2, cb, cr;
	u32 * WiiRAM;
	T_RGB * MyVRAM;
	
	if (HaveFlush != FLUSH_OFF) { return; }
    HaveFlush = FLUSH_VRAM;	
	WiiRAM = VideoRAM[VRAM_Page]; 
	MyVRAM = VRAM;
	
	for (u32 rows = 0; rows < vmode->xfbHeight; rows++)
	{
		for (u32 cols = 0; cols < (vmode->fbWidth >> 1); cols++)
		{
			y1  = (299    * MyVRAM[0].R + 587   * MyVRAM[0].G + 114   * MyVRAM[0].B) / 1000;
			cb1 = (-16874 * MyVRAM[0].R - 33126 * MyVRAM[0].G + 50000 * MyVRAM[0].B + 12800000) / 100000;
			cr1 = (50000  * MyVRAM[0].R - 41869 * MyVRAM[0].G - 8131  * MyVRAM[0].B + 12800000) / 100000;
			
			y2  = (299    * MyVRAM[1].R + 587   * MyVRAM[1].G + 114   * MyVRAM[1].B) / 1000;
			cb2 = (-16874 * MyVRAM[1].R - 33126 * MyVRAM[1].G + 50000 * MyVRAM[1].B + 12800000) / 100000;
			cr2 = (50000  * MyVRAM[1].R - 41869 * MyVRAM[1].G - 8131  * MyVRAM[1].B + 12800000) / 100000;
			
			cb = (cb1 + cb2) >> 1;
			cr = (cr1 + cr2) >> 1;
			
			*WiiRAM = (y1 << 24) | (cb << 16) | (y2 << 8) | cr;			
			
			WiiRAM += 1;
			MyVRAM += 2;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////

void C_GFX::End ()
{
	if (HaveFlush == FLUSH_OFF) { Flush(); }
	
	HaveFlush = FLUSH_VIDEORAM;
	VIDEO_WaitVSync ();
	if (vmode->viTVMode&VI_NON_INTERLACE) { VIDEO_WaitVSync(); }
};

///////////////////////////////////////////////////////////////////////////////

void C_GFX::PostRetraceCallback (u32 Count)
{
	if (HaveFlush != FLUSH_VIDEORAM) { return; }
	VIDEO_SetNextFramebuffer (VideoRAM[VRAM_Page]);  
	VIDEO_Flush ();
	HaveFlush = FLUSH_FULL;
};

///////////////////////////////////////////////////////////////////////////////

void C_GFX::WaitVSync ()
{
	VIDEO_WaitVSync ();
	if (vmode->viTVMode&VI_NON_INTERLACE) { VIDEO_WaitVSync(); }
};

///////////////////////////////////////////////////////////////////////////////
// Fonctions de gestion graphique                                            //
///////////////////////////////////////////////////////////////////////////////

void C_GFX::Clear(const T_RGB Color)
{
	//VIDEO_ClearFrameBuffer (vmode, VRAM, Color);
	for (long int Boucle = 0; Boucle < (vmode->fbWidth * vmode->xfbHeight); Boucle++)
	{
		VRAM[Boucle] = Color;
	}
};

///////////////////////////////////////////////////////////////////////////////

u32 C_GFX::RGBtoYUY2 (T_RGBA RGB1, T_RGBA RGB2)
{
	int y1, cb1, cr1, y2, cb2, cr2, cb, cr;

	y1  = (299 * RGB1.R + 587 * RGB1.G + 114 * RGB1.B) / 1000;
	cb1 = (-16874 * RGB1.R - 33126 * RGB1.G + 50000 * RGB1.B + 12800000) / 100000;
	cr1 = (50000 * RGB1.R - 41869 * RGB1.G - 8131 * RGB1.B + 12800000) / 100000;

	y2  = (299 * RGB2.R + 587 * RGB2.G + 114 * RGB2.B) / 1000;
	cb2 = (-16874 * RGB2.R - 33126 * RGB2.G + 50000 * RGB2.B + 12800000) / 100000;
	cr2 = (50000 * RGB2.R - 41869 * RGB2.G - 8131 * RGB2.B + 12800000) / 100000;

	cb = (cb1 + cb2) >> 1;
	cr = (cr1 + cr2) >> 1;

	return (y1 << 24) | (cb << 16) | (y2 << 8) | cr;
}


///////////////////////////////////////////////////////////////////////////////

int C_GFX::ShowImage (T_Image Image, int X, int Y, u32 ImageId)
{
	u32 rows, cols;
  
	if (Image.Image == NULL) { return ERROR_NULL_VALUE; }
	
	if (ImageId >= Image.NbImages) 
	{ 
		ImageId = 0;  
		return ERROR_BAD_PARAM; 
	}
	
	if (HaveFlush == false)  
	{
		T_RGB  * VBuffer;
		T_RGBA * Vimage;
		
		VBuffer = VRAM + (Y * vmode->fbWidth) + X;
		Vimage = Image.Image + (Image.Height * Image.Width * ImageId);
		
		for (rows = 0; rows < Image.Height; rows++)
		{
			for (cols = 0; cols < Image.Width; cols++)
			{
				if (Vimage->A == 0xFF)
				{ VBuffer[cols] = *((T_RGB*)Vimage); }
				else if (Vimage->A != 0x00)
				{
					VBuffer[cols].R = (VBuffer[cols].R * (255 - Vimage->A)) / 255;
					VBuffer[cols].G = (VBuffer[cols].G * (255 - Vimage->A)) / 255;
					VBuffer[cols].B = (VBuffer[cols].B * (255 - Vimage->A)) / 255;
					
					VBuffer[cols].R += (Vimage->R * Vimage->A) / 255;
					VBuffer[cols].G += (Vimage->G * Vimage->A) / 255;
					VBuffer[cols].B += (Vimage->B * Vimage->A) / 255;						
				}
				
				Vimage++;
			}
			
			VBuffer += vmode->fbWidth;
		}
	}
	else 
	{ 
		u32 MaxBufferAdr, fboffset, fbwidth, Buffer; 
		u32 *VBuffer;
		T_RGBA *bgr;
		
		VBuffer = VideoRAM[VRAM_Page]; 
		bgr = Image.Image + (Image.Height * Image.Width * ImageId);
		fbwidth = Image.Width;
		if (fbwidth & 1) fbwidth++;
		fboffset = X/2;
		fboffset += (Y * (vmode->fbWidth/2));
		MaxBufferAdr = (vmode->fbWidth/2) * vmode->xfbHeight;
		
		for (rows = 0; rows < Image.Height; rows++)
		{
			for (cols = 0; cols < (fbwidth >> 1); cols++)
			{
				if ( ((fboffset + cols) <= MaxBufferAdr) &&  (X+(cols*2) < (vmode->fbWidth) ) &&
				( (X + (2*cols)) >= 0) && ( (Y + rows) >= 0) )
					{
						if ((bgr[0].A == 0xFF) && (bgr[1].A == 0xFF))
					{
						VBuffer[fboffset + cols] = RGBtoYUY2 (bgr[0], bgr[1]); 
					}
					else if ((bgr[0].A == 0xFF) && (bgr[1].A == 0x00))
					{
						Buffer = RGBtoYUY2 (bgr[0], bgr[1]);
						VBuffer[fboffset + cols] = (VBuffer[fboffset + cols] & 0x0080FF8F) | (Buffer & 0xFF8F0080);
					}
					else if ((bgr[0].A == 0x00) && (bgr[1].A == 0xFF))
					{
						Buffer = RGBtoYUY2 (bgr[0], bgr[1]);
						VBuffer[fboffset + cols] = (VBuffer[fboffset + cols] & 0xFF8F0080) | (Buffer & 0x0080FF8F );
					}
				}
				
				bgr += 2;
			}
			
			fboffset += (vmode->fbWidth/2);
		}
	}
	
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_GFX::SetImageAlphaColor (T_Image Image, T_RGB Color)
{
	u32 Boucle;

	if (Image.Image == NULL) { return ERROR_NULL_VALUE; }
	for (Boucle = 0; Boucle < (Image.Width * Image.Height * Image.NbImages); Boucle++)
	{
		if ( (Image.Image[Boucle].R == Color.R) && (Image.Image[Boucle].G == Color.G) &&
		    (Image.Image[Boucle].B == Color.B) )
		{ Image.Image[Boucle].A = 0x00; } 
		else { Image.Image[Boucle].A = 0xFF; }
	}
   
   return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_GFX::SetImageAlphaMask (T_Image Image, T_Image Mask)
{
	u32 Boucle1, Boucle2;
   
	if (Image.Image  == NULL)        { return ERROR_NULL_VALUE; }
	if (Mask.Image   == NULL)        { return ERROR_NULL_VALUE; }
	if (Image.Height != Mask.Height) { return ERROR_BAD_DATA;   }
	if (Image.Width  != Mask.Width)  { return ERROR_BAD_DATA;   }

	for (Boucle1 = 0; Boucle1 < (Image.NbImages); Boucle1++)
	{
		for (Boucle2 = 0; Boucle2 < (Image.Width * Image.Height); Boucle2++)
		{
			Image.Image[Boucle2].A = Mask.Image[Boucle2].A;
		}
		
		Image.Image += Image.Width * Image.Height;
	}
	
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_GFX::SetNbrImagesByHeight (T_Image * Image, u32 Height)
{

	if (Image == NULL) { return ERROR_NULL_VALUE; }
	if (Image->NbImages == 0) { return ERROR_BAD_DATA; }
	if ( ((Image->Height * Image->NbImages) % Height) != 0) { return ERROR_BAD_DATA; }
	if (Image->NbImages > 1) { Image->Height *= Image->NbImages; }
	Image->NbImages = Image->Height / Height;
	Image->Height /= Image->NbImages;
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_GFX::SetNbrImagesByNbr (T_Image * Image, u32 Nbr)
{
	if (Image == NULL) { return ERROR_NULL_VALUE; }
	if (Image->NbImages == 0) { return  ERROR_BAD_DATA; }
	if ( ((Image->Height * Image->NbImages) % Nbr) != 0) { return ERROR_BAD_DATA; }
	if (Image->NbImages > 1) { Image->Height *= Image->NbImages; }
	Image->NbImages = Nbr;
	Image->Height  /= Nbr;
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_GFX::FusionImages (T_Image * Dest, u32 DestId, T_Image Source1, u32 Source1Id, T_Image Source2, u32 Source2Id)
{
	T_RGBA * Src1;
	T_RGBA * Src2;
	T_RGBA * DestImg;
	u32 Boucle1, Boucle2 , X, Y;

	if (Dest == NULL)						{ return ERROR_NULL_VALUE; }
	if (Dest->Image == NULL)				{ return ERROR_NULL_VALUE; }
	if (Source1.NbImages == 0)				{ return ERROR_BAD_DATA;   }
	if (Source1.NbImages < (Source1Id+1)) 	{ return ERROR_BAD_DATA;   }
	if (Source1.Image == NULL)				{ return ERROR_NULL_VALUE; }
	if (Source2.NbImages == 0)				{ return ERROR_BAD_DATA;   }
	if (Source2.NbImages < (Source2Id+1))	{ return ERROR_BAD_DATA;   }
	if (Source2.Image == NULL)				{ return ERROR_NULL_VALUE; }

	X = (Source1.Width  - Source2.Width)  / 2;
	Y = (Source1.Height - Source2.Height) / 2;

	Src1 = Source1.Image  + (Source1.Height*Source1.Width*Source1Id);
	Src2 = Source2.Image  + (Source2.Height*Source2.Width*Source2Id);
	DestImg = Dest->Image + (Source1.Height*Source1.Width*DestId);
	
	for (Boucle1 = 0; Boucle1 < (Source1.Height); Boucle1++)
	{
		for (Boucle2 = 0; Boucle2 < (Source1.Width); Boucle2++)
		{
			if (Boucle1 < Y) { *DestImg = *Src1; }
			else if (Boucle1 >= Source2.Height + Y) { *DestImg = *Src1; }
			else if (Boucle2 < X) { *DestImg = *Src1; }
			else if (Boucle2 < Source2.Width + X)
			{
				if (Src2->A == 0) 
				{ *DestImg = *Src1; }
				else if (Src2->A == 0xFF) 
				{ *DestImg = *Src2; }
				else
				{
					DestImg->R = (Src1->R * (255 - Src2->A)) / 255;
					DestImg->G = (Src1->G * (255 - Src2->A)) / 255;
					DestImg->B = (Src1->B * (255 - Src2->A)) / 255;
					
					DestImg->R += (Src2->R * Src2->A) / 255;
					DestImg->G += (Src2->G * Src2->A) / 255;
					DestImg->B += (Src2->B * Src2->A) / 255;				
				}
				
				Src2++;
			}
			else
			{ *DestImg = *Src1; }
			Src1++;
			DestImg++;
		}
	}

	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////

int C_GFX::FusionImages (T_Image * Dest, T_Image Source1, u32 Source1Id, T_Image Source2, u32 Source2Id, u8 Fade)
{
	T_RGBA * Src1;
	T_RGBA * Src2;
	u16 R, G, B;
	u32 Boucle;

	if (Dest == NULL)						{ return ERROR_NULL_VALUE; }
	if (Dest->Image == NULL)				{ return ERROR_NULL_VALUE; }
	if (Source1.NbImages == 0)				{ return ERROR_BAD_DATA;   }
	if (Source1.NbImages < (Source1Id+1)) 	{ return ERROR_BAD_DATA;   }
	if (Source1.Image == NULL)				{ return ERROR_NULL_VALUE; }
	if (Source2.NbImages == 0)				{ return ERROR_BAD_DATA;   }
	if (Source2.NbImages < (Source2Id+1))	{ return ERROR_BAD_DATA;   }
	if (Source2.Image == NULL)				{ return ERROR_NULL_VALUE; }
	if (Source1.Height != Source2.Height)	{ return ERROR_BAD_DATA;   }
	if (Source1.Width  != Source2.Width)	{ return ERROR_BAD_DATA;   }
	if (Source1.Height != Dest->Height)		{ return ERROR_BAD_DATA;   }
	if (Source1.Width  != Dest->Width)		{ return ERROR_BAD_DATA;   }


	Src1 = Source1.Image + (Source1.Height*Source1.Width*Source1Id);
	Src2 = Source2.Image + (Source2.Height*Source2.Width*Source2Id);
	
	for (Boucle = 0; Boucle < (Source1.Height*Source1.Width); Boucle++)
	{
		if      (Fade == 0)   { Dest->Image[Boucle] = Src1[Boucle]; }
		else if (Fade == 255) { Dest->Image[Boucle] = Src2[Boucle]; }
		else
		{
			R = (Src1[Boucle].R * (255 - Fade)) / 255;
			G = (Src1[Boucle].G * (255 - Fade)) / 255;
			B = (Src1[Boucle].B * (255 - Fade)) / 255;
			
			R += (Src2[Boucle].R * Fade) / 255;
			G += (Src2[Boucle].G * Fade) / 255;
			B += (Src2[Boucle].B * Fade) / 255;
			
			Dest->Image[Boucle].R = R;
			Dest->Image[Boucle].G = G;
			Dest->Image[Boucle].B = B;
		}
		
		Dest->Image[Boucle].A = 0XFF;
	}
   
	return NO_ERROR;
}

///////////////////////////////////////////////////////////////////////////////
// Fonctions de gestion du texte                                             //
///////////////////////////////////////////////////////////////////////////////

u32 C_GFX::ShowCaractere (T_Image &Font, const char Cara, int X, int Y, T_RGB Color, bool Display)
{
	u32 fboffset, CharSize, rows, cols;
	T_RGBA *bgr;

	CharSize = 0;
	if (((X + Font.Width) > vmode->fbWidth) || ((Y + Font.Height) > vmode->xfbHeight)) return 0;
	fboffset = X + (Y * vmode->fbWidth);
	bgr = Font.Image;
	bgr += Cara * Font.Width * Font.Height;

	for (rows = 0; rows < Font.Height; rows++)
	{
		for (cols = 0; cols < Font.Width; cols++)
		{
			if (bgr->A != 0x00)
			{
				if (Display == true)
				{
					VRAM[fboffset + cols].R = Color.R;
					VRAM[fboffset + cols].G = Color.G;
					VRAM[fboffset + cols].B = Color.B;
				}
				
				if ( CharSize < ((cols) + 1) ) { CharSize = (cols) + 1; }
			}
			
			bgr ++;
		}
		
		fboffset += (vmode->fbWidth);
	}

	return CharSize;
}

///////////////////////////////////////////////////////////////////////////////

void C_GFX::ShowString (const char * Cara, int X, int Y, T_RGB Color)
{
	u32 CharSize;
	int CurrentX = X;
	int CurrentY = Y;
	
	while (Cara[0] != 0x00)
	{
		if (Cara[0] == '\n')
		{
			CurrentX = X;
			CurrentY += ImageRessource[ImageRessource_Font].Height;
		}
		else if (Cara[0] != '\r')
		{
			CharSize = ShowCaractere (ImageRessource[ImageRessource_Font], Cara[0], CurrentX, CurrentY, Color);
			
			if (CharSize == 0) 
			{ CurrentX += ImageRessource[ImageRessource_Font].Width / 2 ; } 
			else 
			{ CurrentX += CharSize; }
			
			if ((CurrentX + ImageRessource[ImageRessource_Font].Width) > vmode->fbWidth)
			{
				CurrentX = X;
				CurrentY += ImageRessource[ImageRessource_Font].Height;
			}
		}
		
		Cara++;
	}
}

///////////////////////////////////////////////////////////////////////////////

u32 C_GFX::SizeOfString (const char * Cara)
{

	u32 CurrentX = 0;
	u32 CurrentY = 0;
	u32 CharSize, BigCharSize = 0;

	while (Cara[0] != 0x00)
	{
		if (Cara[0] == '\n')
		{
			CurrentX = 0;
			CurrentY += ImageRessource[ImageRessource_Font].Height;
		}
		else if (Cara[0] != '\r')
		{
			CharSize = ShowCaractere (ImageRessource[ImageRessource_Font], Cara[0], CurrentX, CurrentY, RGB_BLACK, false);
			
			if (CharSize == 0) 
			{ CurrentX += ImageRessource[ImageRessource_Font].Width / 2; } 
			else 
			{ CurrentX += CharSize; }
			
			if ((CurrentX + ImageRessource[ImageRessource_Font].Width) > vmode->fbWidth)
			{
				CurrentX = 0;
				CurrentY += ImageRessource[ImageRessource_Font].Height;
			}
		}
		
		if (BigCharSize < CurrentX) { BigCharSize = CurrentX; }
		Cara++;
	}

	return BigCharSize;
}

///////////////////////////////////////////////////////////////////////////////
