///////////////////////////////////////////////////////////////////////////////
// Resources file                                                            //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <gccore.h>
#include <ogc/conf.h>

#include "Wii_gfx.hpp"
#include "Ressources.h"
#include "AppConfig.h"
#include "T_error.h"
#include "FS.hpp"

///////////////////////////////////////////////////////////////////////////////

const char * const textRessource_En [TextRessource_COUNT] =
{
	"No Homebrew found",
	"Homebrew menu",
	"Return",
	"Wii menu",
	"Shutdown Wii",
	"Restart loader",
	"Homebrew",
	"Applications",
	"Emulators",
	"Themes",
	"Default theme",
	"Wii Channels",
	"Press (A)",
	"Invalid executable file",
	"Reception in progress...",
	"Error data reception",
	"No files found",
	"Version",
	"GameCube Homebrew",
	"Loading homebrew. Prepare your GameCube pad. To exit homebrew and return to the Wii Menu press the button \"Power\" on the front of the console."
};

const char * const textRessource_Ge [TextRessource_COUNT] =
{
	"Keine Homebrew gefunden",
	"Homebrew-Menu",
	"Zuruck",
	"Wii-Menu",
	"Ausschalten Wii",
	"Starten Sie Loader",
	"Homebrew",
	"Anwendungen",
	"Emulatoren",
	"Themen",
	"Standard-Theme",
	"Wii-Kanle",
	"Drucken Sie (A)",
	"Ungltige ausfhrbare Datei",
	"Eingang in...",
	"Fehler beim Empfangen von Daten",
	"Keine Dateien gefunden",
	"Version",
	"GameCube Homebrew",
	"Lade homebrew. Bereiten Sie Ihren GameCube Pad. Um das Men zu verlassen Homebrew und zurck auf die Wii-Men drcken Sie die Taste \"Power\" auf der Vorderseite der Konsole."
};

const char * const textRessource_Fr [TextRessource_COUNT] =
{
	"Aucun Homebrew trouv",
	"Menu Homebrew",
	"Retour",
	"Menu Wii",
	"Eteindre Wii",
	"Redmarrer Loader",
	"Homebrew",
	"Applications",
	"Emulateurs",
	"Thmes",
	"Thme par dfaut",
	"Chanes Wii",
	"Appuyez sur (A)",
	"Excutable invalide",
	"Rception en cours...",
	"Erreur de rception des donnes",
	"Aucun fichier trouv",
	"Version",
	"Homebrew GameCube",
	"Chargement du homebrew en cours. Prparez votre manette GameCube. Pour quitter le homebrew et revenir au menu Wii appuyez sur le boutton \"Power\" sur la face avant de la console."
};

const char * const textRessource_Sp [TextRessource_COUNT] =
{
	"Homebrew no encontrado",
	"Menu Homebrew",
	"Volver",
	"Menu Wii",
	"Apagar Wii",
	"Reiniciar loader",
	"Homebrew",
	"Aplicaciones",
	"Emuladores",
	"Temas",
	"Tema por defecto",
	"Canales Wii",
	"Prensa (A)",
	"Archivo ejecutable no vlido",
	"Recepcin en el progreso...",
	"Error de recepcin de datos",
	"No archivos encontrados",	
	"Version",
	"GameCube Homebrew",	
	"Cargando homebrew. Prepara tu GameCube almohadilla. Para salir de homebrew y volver al Men de Wii pulse en el botn \"Power\" en la parte frontal de la consola."
};

const char * const textRessource_It [TextRessource_COUNT] =
{
	"Nessnun Homebrew trovato",
	"Homebrew menu",
	"Ritorna al Menu",
	"Riavvia il wii",
	"Spegni il Wii",
	"Riavvia il Loader",
	"Homebrew",
	"Applicazioni",
	"Emulatori",
	"Temi",
	"Tema di default",
	"Canali Wii",
	"Premere (A)",
	"Non valide file eseguibile",
	"Ricezione in corso...",
	"Errore durante la ricezione dei dati",
	"Nessun file trovato",
	"Versione",
	"GameCube Homebrew",
	"Caricamento homebrew. Preparare il pad GameCube. Homebrew per uscire e ritornare al menu Wii premere il tasto \"Power\" sulla parte anteriore della console."
};

const char * const textRessource_Du [TextRessource_COUNT] =
{
	"Geen Homebrew gevonden",
	"Homebrew menu",
	"Terug",
	"Wii menu",
	"Afsluiten Wii",
	"Herstart loader",
	"Homebrew",
	"Toepassingen",
	"Emulators",
	"Thema's",
	"Standaard thema",
	"Wii-kanalen",
	"Druk op (A)",
	"Ongeldige uitvoerbaar bestand",
	"Receptie in de vooruitgang...",
	"Fout gegevens receptie",
	"Geen bestanden gevonden",
	"Versie",
	"GameCube Homebrew",
	"Laden Homebrew. Bereid je GameCube pad. Homebrew te verlaten en terug te keren naar de Wii Menu druk op de knop \"Power\" op de voorzijde van de console."
};

///////////////////////////////////////////////////////////////////////////////

const T_TextErrors TextErrors [] =
{
	{ ERROR_UNKNOW,				"Unknow error"					},
	{ ERROR_NULL_VALUE, 		"Null value"					},
	{ ERROR_BAD_PARAM, 			"Bad parameter"					},
	{ ERROR_BAD_DATA,			"Bad data"						},
	{ ERROR_OUT_OF_MEMORY, 		"Out of memory"					},
	{ ERROR_SYSTEM, 			"System error"					},
	
	{ FS_ERROR_IO, 				"I-O error"						},
	{ FS_ERROR_DEVICE_NOT_FOUND,"Device not found"				},
	{ FS_ERROR_NOT_OPENED,		"Not opened"					},
	{ FS_ERROR_FILE_OPENED,		"File always opended"			},
	{ FS_ERROR_NOT_FOUND,		"Not found"						},
	{ FS_ERROR_NOT_EXECUTED,	"Not executed" 					},
	{ FS_ERROR_CANT_LOAD_FILE,	"Can't load file"				},
	{ FS_ERROR_EXIST,			"File or directory exist"		},
	{ FS_ERROR_NOT_EXIST,		"File or directory not exist"	},
	{ FS_ERROR_READ_ONLY,		"File or directory is read only"},
	{ FS_ERROR_OUT_OF_SPACE,	"Out of space"					},
	{ FS_ERROR_NOT_DIRECTORY,	"Not a directory"				},
	{ FS_ERROR_NOT_FILE,		"Not a file"					},
	{ FS_ERROR_NOT_SUPPORTED,	"Not supported"					},
	
	{ ES_EINVAL,				"System error"					},
	{ ES_ENOTINIT,				"ES initialisation error"		},
	
	{ -EPERM,					"Not super-user"				},
	{ -ENOENT,					"No such file or directory"		},
	{ -ESRCH, 					"No such process"				},
	{ -EINTR,					"Interrupted system call"		},
	{ -EIO,						"I/O error"						},
	{ -ENXIO,					"No such device or address"		},
	{ -E2BIG, 					"Arg list too long"				},
	{ -ENOEXEC,					"Exec format error"				},
	{ -EBADF,					"Bad file number"				},
	{ -ECHILD, 					"No children"					},
	{ -EAGAIN,					"No more processes"				},
	{ -ENOMEM, 					"Not enough core"				},
	{ -EACCES,					"Permission denied"				},
	{ -EFAULT,					"Bad address"					},
	{ -ENOTBLK,					"Block device required"			},
	{ -EBUSY,					"Mount device busy"				},
	{ -EEXIST,					"File exists"					},
	{ -EXDEV, 					"Cross-device link"				},
	{ -ENODEV,					"No such device"				},
	{ -ENOTDIR,					"Not a directory"				},
	{ -EISDIR,					"Is a directory"				},
	{ -EINVAL,					"Invalid argument"				},
	{ -ENFILE,					"Too many open files in system"	},
	{ -EMFILE, 					"Too many open files"			},
	{ -ENOTTY,					"Not a typewriter"				},
	{ -ETXTBSY,					"Text file busy"				},
	{ -EFBIG,					"File too large"				},
	{ -ENOSPC,					"No space left on device"		},
	{ -ESPIPE,					"Illegal seek"					},
	{ -EROFS,					"Read only file system"			},
	{ -EMLINK,					"Too many links"				},
	{ -EPIPE,					"Broken pipe"					},
	{ -EDOM,					"Math arg out of domain of func"},
	{ -ERANGE,					"Math result not representable"	},
	{ -ENOMSG,					"No message of desired type"	},
	{ -EIDRM, 					"Identifier removed"			},
	{ -ECHRNG,					"Channel number out of range"	},
	{ -EL2NSYNC,				"Level 2 not synchronized"		},
	{ -EL3HLT,					"Level 3 halted"				},
	{ -EL3RST, 					"Level 3 reset"					},
	{ -ELNRNG, 					"Link number out of range"		},
	{ -EUNATCH,					" Protocol driver not attached"	},
	{ -ENOCSI,					"No CSI structure available"	},
	{ -EL2HLT,					"Level 2 halted"				},
	{ -EDEADLK,					"Deadlock condition"			},
	{ -ENOLCK,					"No record locks available"		},
	{ -EBADE,					"Invalid exchange"				},
	{ -EBADR,					"Invalid request descriptor"	},
	{ -EXFULL,					"Exchange full"					},
	{ -ENOANO,					"No anode"						},
	{ -EBADRQC,					"Invalid request code"			},
	{ -EBADSLT,					"Invalid slot"					},
	{ -EDEADLOCK,				"File locking deadlock error"	},
	{ -EBFONT, 					"Bad font file fmt"				},
	{ -ENOSTR,					"Device not a stream"			},
	{ -ENODATA,					"No data (for no delay io)"		},
	{ -ETIME,					"Timer expired"					},
	{ -ENOSR, 					"Out of streams resources"		},
	{ -ENONET,					"Machine is not on the network"	},
	{ -ENOPKG,					"Package not installed"			},
	{ -EREMOTE,					"The object is remote"			},
	{ -ENOLINK,					"The link has been severed"		},
	{ -EADV,					"Advertise error"				},
	{ -ESRMNT,					"Srmount error"					},
	{ -ECOMM,					"Communication error on send"	},
	{ -EPROTO,					"Protocol error"				},
	{ -EMULTIHOP,				"Multihop attempted"			},
	{ -ELBIN,					"Inode is remote (not really error)"	},
	{ -EDOTDOT,					"Cross mount point (not really error)"	},
	{ -EBADMSG,					"Trying to read unreadable message"		},
	{ -EFTYPE,					"Inappropriate file type or format"		},
	{ -ENOTUNIQ,				"Given log. name not unique"			},
	{ -EBADFD,					"f.d. invalid for this operation"		},
	{ -EREMCHG,					"Remote address changed"				},
	{ -ELIBACC,					"Can't access a needed shared lib"		},
	{ -ELIBBAD ,				"Accessing a corrupted shared lib"		},
	{ -ELIBSCN,					".lib section in a.out corrupted"		},
	{ -ELIBMAX,					"Attempting to link in too many libs"	},
	{ -ELIBEXEC,				"Attempting to exec a shared library"	},
	{ -ENOSYS,					"Function not implemented"				},
	{ -ENMFILE,					"No more files"							},
	{ -ENOTEMPTY,				"Directory not empty"					},
	{ -ENAMETOOLONG,			"File or path name too long"			},
	{ -ELOOP,					"Too many symbolic links"				},
	{ -EOPNOTSUPP,				"Operation not supported on transport endpoint"},
	{ -EPFNOSUPPORT,			"Protocol family not supported"			},
	{ -ECONNRESET,				"Connection reset by peer"				},
	{ -ENOBUFS,					"No buffer space available"				},
	{ -EAFNOSUPPORT,			"Address family not supported by protocol family"},
	{ -EPROTOTYPE,				"Protocol wrong type for socket"		},
	{ -ENOTSOCK,				"Socket operation on non-socket"		},
	{ -ENOPROTOOPT,				"Protocol not available"				},
	{ -ESHUTDOWN,				"Can't send after socket shutdown"		},
	{ -ECONNREFUSED,			"Connection refused"					},
	{ -EADDRINUSE,				"Address already in use"				},
	{ -ECONNABORTED,			"Connection aborted"					},
	{ -ENETUNREACH,				"Network is unreachable"				},
	{ -ENETDOWN,				"Network interface is not configured"	},
	{ -ETIMEDOUT,				"Connection timed out"					},
	{ -EHOSTDOWN,				"Host is down"							},
	{ -EHOSTUNREACH,			"Host is unreachable"					},
	{ -EINPROGRESS,				"Connection already in progress"		},
	{ -EALREADY,				"Socket already connected"				},
	{ -EDESTADDRREQ,			"Destination address required"			},
	{ -EMSGSIZE,				"Message too long"						},
	{ -EPROTONOSUPPORT,			"Unknown protocol"						},
	{ -ESOCKTNOSUPPORT,			"Socket type not supported"				},
	{ -EADDRNOTAVAIL,			"Address not available"					},
	{ -EISCONN,					"Socket is already connected"			},
	{ -ENOTCONN,				"Socket is not connected"				},
	{ -ENOTSUP,					"Not supported"							},
	{ -ENOMEDIUM,				"No medium (in tape drive)"				},
	{ -ENOSHARE,				"No such host or network path"			},
	{ -ECASECLASH,				"Filename exists with different case"	},
	{ -EOVERFLOW,				"Value too large for defined data type"	},
	{ -ECANCELED,				"Operation canceled."					},	
	
	{0, NULL} /* End of errors */
};

///////////////////////////////////////////////////////////////////////////////

T_RGB 	AlphaColor		= { 0xFF, 0x00, 0xFF };
T_RGB 	AlphaMaskColor	= { 0x00, 0xFF, 0x00 };

char SMB_Server		[16] = "";
char SMB_Share		[20] = "";
char SMB_User		[20] = "guest";
char SMB_Password	[20] = "";

char * ThemeDir = NULL;

T_Image ImageRessource[ImageRessource_COUNT];
char * textRessource[TextRessource_COUNT];


#ifdef ENABLE_MP3
u8 SoundPlay = 1;

u8 * SoundRessource[] =
{
    (u8*) mp3Intro,
    (u8*) mp3Loop
};

long SoundRessource_Length [2];

#endif

u8 Common_IsChannel = false;

///////////////////////////////////////////////////////////////////////////////
