///////////////////////////////////////////////////////////////////////////////
// Resources file                                                            //
// Part of Wii Homebrew Launcher                                             //
// Created by Hell Hibou (2008)                                              //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#ifndef _UOBIHLLEH_WII_RESSOURCES_H
#define _UOBIHLLEH_WII_RESSOURCES_H

#ifdef __cplusplus
extern "C" {
#endif

#include "T_image.h"
#include "AppConfig.h"

#define DEFAULT_STRING_PAGE_CODE		String_PageCode_ANSI

#define Default_Background_In_Menu		false

#define DefaultColorRessource_Menu				RGB_WHITE
#define DefaultColorRessource_TextMenu			RGB_GREY
#define DefaultColorRessource_TextBar			RGB_GREY
#define DefaultColorRessource_TextChannel		RGB_GREY
#define DefaultColorRessource_TextMessageBox	RGB_GREY

#define TextRessource_NoHomebrewFound	0
#define TextRessource_HomebrewMenu		1
#define TextRessource_Return			2
#define TextRessource_WiiMenu			3
#define TextRessource_Shutdown			4
#define TextRessource_RestartLoader		5
#define TextRessource_Homebrew			6
#define TextRessource_Applications		7
#define TextRessource_Emulator			8
#define TextRessource_Themes			9
#define TextRessource_DefaultTheme		10
#define TextRessource_WiiChannels		11
#define TextRessource_PressA			12
#define TextRessource_InvalidExe		13
#define TextRessource_ReceptInProgress	14
#define TextRessource_ConnexionError	15
#define TextRessource_NoFilesFound		16
#define TextRessource_Version			17
#define TextRessource_HomebrewGC		18
#define TextRessource_HomebrewGC_Load	19
#define TextRessource_COUNT				20

#define ImageRessource_Background       0
#define ImageRessource_Bar              1
#define ImageRessource_ThumbEmpty       2
#define ImageRessource_ThumbSelect      3
#define ImageRessource_ThumbSprite      4
#define ImageRessource_ThumbUsed        5
#define ImageRessource_ThumbDefault     6
#define ImageRessource_Font             7
#define ImageRessource_Left             8
#define ImageRessource_Right            9
#define ImageRessource_Button1          10
#define ImageRessource_Button2          11
#define ImageRessource_FontLight        12
#define ImageRessource_WiiBar			13
#define ImageRessource_Cursor           14
#define ImageRessource_MessageBox		15
#define ImageRessource_Directory		16
#define ImageRessource_COUNT			17


#ifdef ENABLE_MP3
#define SoundRessource_Intro             0
#define SoundRessource_Loop              1
#endif


typedef struct T_TextErrors
{
	const int		Code;
	const char *	Message;
} T_TextErrors;

extern const T_TextErrors TextErrors[];

extern u8	BkgroundInMenu;

extern T_RGB AlphaColor;
extern T_RGB AlphaMaskColor;
extern char * ThemeDir;

extern char SMB_Server		[16];
extern char SMB_Share		[20];
extern char SMB_User		[20];
extern char SMB_Password	[20];

extern char * textRessource [];

extern const char * const textRessource_En [TextRessource_COUNT];
extern const char * const textRessource_Fr [TextRessource_COUNT];
extern const char * const textRessource_Sp [TextRessource_COUNT];
extern const char * const textRessource_It [TextRessource_COUNT];
extern const char * const textRessource_Du [TextRessource_COUNT];
extern const char * const textRessource_Ge [TextRessource_COUNT];

extern T_Image	ImageRessource[];

extern const u8		Bin_JumpCode[];
extern const long	Bin_JumpCode_Length;
extern const u8		Dol_Reboot[];
extern const u8		Dol_WiiHbLoader[];

extern const u8		pngFsFrontSD[];
extern const long	pngFsFrontSD_Length;
extern const u8		pngFsGecko[];
extern const long	pngFsGecko_Length;
extern const u8		pngFsUSB[];
extern const long	pngFsUSB_Length;
extern const u8		pngFsDVD[];
extern const long	pngFsDVD_Length;
extern const u8		pngFsSMB[];
extern const long	pngFsSMB_Length;

extern const u8		pngTitleWiiChannel[];
extern const long	pngTitleWiiChannel_Length;
extern const u8		pngTitleHomebrew[];
extern const long	pngTitleHomebrew_Length;
extern const u8		pngTitleApps[];
extern const long	pngTitleApps_Length;
extern const u8		pngTitleEmulator[];
extern const long	pngTitleEmulator_Length;
extern const u8		pngTitleHomebrewGC[];
extern const long	pngTitleHomebrewGC_Length;	
extern const u8		pngTitleTheme[];
extern const long	pngTitleTheme_Length;
extern const u8		pngTitleWiiBar[];
extern const long	pngTitleWiiBar_Length;

extern const u8		pngDirectory[];
extern const long	pngDirectory_Length;

extern const u8		pngTitleWiiChannelDefault[];
extern const long	pngTitleWiiChannelDefault_Length;
extern const u8		pngTitleWiiChannelVC[];
extern const long	pngTitleWiiChannelVC_Length;
extern const u8		pngTitleWiiChannelWiiWare[];
extern const long	pngTitleWiiChannelWiiWare_Length;

extern const u8		pngTitleZoneAll[];
extern const long	pngTitleZoneAll_Length;
extern const u8		pngTitleZoneFr[];
extern const long	pngTitleZoneFr_Length;
extern const u8		pngTitleZoneGe[];
extern const long	pngTitleZoneGe_Length;
extern const u8		pngTitleZoneJp[];
extern const long	pngTitleZoneJp_Length;
extern const u8		pngTitleZoneKr[];
extern const long	pngTitleZoneKr_Length;
extern const u8		pngTitleZonePal[];
extern const long	pngTitleZonePal_Length;
extern const u8		pngTitleZoneUs[];
extern const long	pngTitleZoneUs_Length;

extern const u8		pngThumbEmpty[];
extern const long	pngThumbEmpty_Length;
extern const u8		pngTitle[];
extern const long	pngTitle_Length;
extern const u8		pngBackground[];
extern const long	pngBackground_Length;
extern const u8		pngBar[];
extern const long	pngBar_Length;
extern const u8		pngThumbEmpty[];
extern const long	pngThumbEmpty_Length;
extern const u8		pngThumbSelect[];
extern const long	pngThumbSelect_Length;
extern const u8		pngThumbSprite[];
extern const long	pngThumbSprite_Length;
extern const u8 	pngThumbUsed[];
extern const long	pngThumbUsed_Length;
extern const u8		pngThumbDefault[];
extern const long	pngThumbDefault_Length;
extern const u8		pngFont[];
extern const long	pngFont_Length;
extern const u8		pngFontLight[];
extern const long	pngFontLight_Length;
extern const u8		pngLeft[];
extern const long	pngLeft_Length;
extern const u8		pngRight[];
extern const long	pngRight_Length;
extern const u8		pngButton1[];
extern const long	pngButton1_Length;
extern const u8		pngButton2[];
extern const long	pngButton2_Length;
extern const u8		pngButton12[];
extern const long	pngButton12_Length;
extern const u8		pngButtonA[];
extern const long	pngButtonA_Length;
extern const u8		pngCursor[];
extern const long	pngCursor_Length;
extern const u8		pngMessageBox[];
extern const long	pngMessageBox_Length;

#ifdef ENABLE_MP3
extern u8	SoundPlay;
extern u8 *	SoundRessource [];
extern long	SoundRessource_Length[];

extern const u8		mp3Intro[];
extern const long	mp3Intro_Length;
extern const u8		mp3Loop[];
extern const long	mp3Loop_Length;
#endif

#ifdef __cplusplus
}
#endif

extern u8 Common_IsChannel;

#endif

///////////////////////////////////////////////////////////////////////////////
