///////////////////////////////////////////////////////////////////////////////
// Wii System Library                                                        //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <ogc/ipc.h>
#include <stdio.h>
#include <fat.h>

#include "../../AppConfig.h"
#include "../../Common.h"

#include "asm.h"
#include "processor.h"
#include "system.h"
#include "Ressources.h"
#include "config.hpp"
#include "kmemory.h"
#include "T_error.h"

#include "control.hpp"

///////////////////////////////////////////////////////////////////////////////

typedef struct T_IosId
{
	u32	TypeId;
	u16 SystemType;
	u16 IOS;
} __attribute__ ((__packed__)) T_IosId;

///////////////////////////////////////////////////////////////////////////////

int GetInstalledIOS (u8 * IOSlist)
{
	u32 * Content = NULL;
	T_IosId * TitlesID = NULL;
	u32 ChannelsCount ATTRIBUTE_ALIGN(32);

	Content = (u32*) memalign(32, 32);
	if (Content == NULL) { return -1; }
	if (ES_GetNumTitles(&ChannelsCount) < 0) { free(Content); return -1; }
	TitlesID = (T_IosId*)	 memalign(32, sizeof(u64) * ChannelsCount);
	if (TitlesID == NULL)	{ free(Content); return -1; }
	memset (TitlesID, 0x0LL, sizeof(u64) * ChannelsCount);
	if (ES_GetTitles((u64*)TitlesID, ChannelsCount)  < 0) 
	{
		free(Content); 
		free(TitlesID);
		return -1;
	}
	
	for (u32 Boucle = 0; Boucle < 256; Boucle++) { IOSlist[Boucle] = 0x00; }
	for (u32 Boucle = 0; Boucle < ChannelsCount; Boucle++) 
	{
		if ((TitlesID[Boucle].TypeId == 0x00000001L) && (TitlesID[Boucle].SystemType == 0x0000) )
		{ IOSlist[TitlesID[Boucle].IOS] = 0x01; }
	}
	
	free(Content); 
	free(TitlesID);
	return 0;
}

///////////////////////////////////////////////////////////////////////////////

s32 GetPreferedIOS ()
{
	int RetVal;
	u8 IOSlist [256];
	
	RetVal = GetInstalledIOS (IOSlist);
	if (RetVal < 0) return RetVal;
	if (IOSlist[16] == 1) return 16;

	RetVal = IOS_GetVersion();
	if (RetVal > 9) return RetVal;
	
	for (int Boucle = 36; Boucle > 0; Boucle--)
	{
		if (IOSlist[Boucle] == 1) return Boucle;
	}
	
	return -1;
}

///////////////////////////////////////////////////////////////////////////////

int LaunchTitle (u64 TitleID)
{
	static tikview viewdata[0x10] ATTRIBUTE_ALIGN(32);
	int RetVal;
	u32 views;

	RetVal = ES_GetNumTicketViews(TitleID, &views);
	if (RetVal < 0)	{ return -1; }
	if (!views)		{ return -1; }
	if (views > 16)	{ return -1; }

	RetVal = ES_GetTicketViews (TitleID, viewdata, views);

	switch (RetVal)
	{
		case ES_EINVAL:
			return ERROR_BAD_DATA;
			
		case ES_ENOMEM: 
			return ERROR_OUT_OF_MEMORY;
			
		case ES_ENOTINIT:
			return ERROR_SYSTEM;
		
		case ES_EALIGN:
			return ERROR_SYSTEM;
	}
	
	ES_LaunchTitle(TitleID, &viewdata[0]);
	
	return ERROR_SYSTEM;
}

///////////////////////////////////////////////////////////////////////////////

void Reload()
{
	if (Common_IsChannel == true)
	{
		if (LaunchTitle(REBOOT_TITLE_ID) < 0) { LaunchTitle(CHANNEL_ID_SYSTEM); }
	}
	else
	{
		void (*reload)() = (void(*)())0x80001800;
		reload();	
	}
}

///////////////////////////////////////////////////////////////////////////////

void SetPowerMode (int Value)
{
	int fd = -1;

	if (!((Value > 0x2000) && (Value < 0x2006) ) ) { return; }
	fd = IOS_Open("/dev/stm/immediate",0);
	IOS_Ioctl(fd, Value, NULL, 0, NULL, 0);
	IOS_Close(fd);
}

///////////////////////////////////////////////////////////////////////////////

void SetLED (u32 LedMode, u32 LedIntensite)
{
	u32 Data[0x20] __attribute__((aligned(32)));
	int f_led = -1;
	int RetVal;

	f_led = IOS_Open("/dev/stm/immediate",0);
	if (f_led == 0) { return; }

	Data[0] = LedMode;
	RetVal = IOS_Ioctl(f_led, 0x6002, Data, 4, NULL, 0);

	Data[0] = LedIntensite;
	RetVal = IOS_Ioctl(f_led, 0x6001, Data, 4, NULL, 0);

	IOS_Close(f_led);
}

///////////////////////////////////////////////////////////////////////////////
