///////////////////////////////////////////////////////////////////////////////
// Class C_App                                                               //
// Created by Hell Hibou (2008)                                              //
// Part of Wii Homebrew Launcher                                             //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////


#include <string.h>

#include "../../common.h"
#include "../../AppConfig.h"

#include "T_error.h"
#include "asm.h"
#include "processor.h"
#include "app.hpp"

#ifndef APP_MODE_LITE
	#include "ressources.h"
	#include "kmemory.h"
	#include "system.h"
#endif


#define MAGIC_argv				0x5f617267
#define TEXT_HEADER_BLOC_BEGIN	0x48

extern void BeforeExit  ();

typedef struct T_DolHeader 
{
	u32 text_pos	[7];
	u32 data_pos	[11];
	u32 text_start	[7];
	u32 data_start	[11];
	u32 text_size	[7];
	u32 data_size	[11];
	u32 bss_start;
	u32 bss_size;
	u32 entry_point;
} T_DolHeader;


typedef struct T_libogc_argv_data // LibOGC internal argv structure
{
	int argvMagic;		// argv magic number, set to 0x5f617267 ('_arg') if valid 
	char *commandLine;	// base address of command line, set of null terminated strings
	int length;			// total length of command line
	int argc;
	char **argv;
	char **endARGV;
} T_libogc_argv_data;	


typedef struct T_libogc_argv
{
	u8		Code;		// = 0x48
	u8		Size1;		// Hi  bits of size of text area
	u16		Size2;		// Low bits of size of text area
	u32		argvMagic;	// argv magic number, set to 0x5f617267 ('_arg') if valid 
	T_libogc_argv_data Data;
} T_libogc_argv;

///////////////////////////////////////////////////////////////////////////////

#ifndef APP_MODE_LITE

int C_App::SetLibogcArg (const void * Data, const char * CmdLine)
{
	char * MemData;
	T_libogc_argv * argv = (T_libogc_argv *) Data;
	char * CharPtr;
	u32 Size;
	char * * Argv;
	
	if (Data == NULL)		{ return ERROR_NULL_VALUE; }
	if (CmdLine == NULL)	{ return ERROR_NULL_VALUE; }
	if (argv->argvMagic != MAGIC_argv)			{ return ERROR_BAD_DATA;}
	if (argv->Code != TEXT_HEADER_BLOC_BEGIN)	{ return ERROR_BAD_DATA;}
	if ( ((argv->Size1 * 0x10000) + argv->Size2) < sizeof(T_libogc_argv) ) { return ERROR_BAD_DATA; }

	Size = strlen (CmdLine) + 3;
	CharPtr = (char *) CmdLine;
	
	while (CharPtr[0] != 0x00)
	{
		if (CharPtr[0] == '\t') { Size += sizeof (char*); }
		CharPtr++;
	}
	
	MemData = (char*) MEM2_malloc(Size);
	if (MemData == NULL)	{ return ERROR_OUT_OF_MEMORY; }
	strcpy (MemData, CmdLine);
	MemData[strlen (CmdLine)+1] = 0x00;
	MemData[strlen (CmdLine)+2] = 0x00;
	CharPtr = MemData;
	argv->Data.endARGV = Argv = (char**) &MemData[strlen (CmdLine) + 3];
	argv->Data.argvMagic	= MAGIC_argv;
	argv->Data.commandLine	= MemData;
	argv->Data.length = strlen(CmdLine)+1;
	argv->Data.argc	= 1;
	argv->Data.argv = Argv;
	
	while (*CharPtr != 0x00)
	{
		if (*CharPtr == '\t') 
		{ 
			*CharPtr = 0;
			*Argv = &CharPtr[-1];
			Argv++;
			argv->Data.argc	++;
			argv->Data.endARGV ++;
		}
		CharPtr++;
	}

	return NO_ERROR;
}

#endif

///////////////////////////////////////////////////////////////////////////////

#ifndef APP_MODE_LITE

int C_App::Launch (const void * Data, const char * CmdLine)
{
	void (*EntryPoint)();
	T_WHL_Common_Ressources * WHL_Common;
	u32 Level = 0;
	
	if (this->Load(Data, CmdLine, APP_ACTION_PRELOAD)  == NULL) { return ERROR_BAD_DATA; }
	
	WHL_Common = GET_GLOBAL_PARAM_ADRESS;
	INIT_WHL_Common_Ressources(WHL_Common);
	
	// Install reboot Code if channel Mode
	if (Common_IsChannel == true) 
	{
		ICInvalidateRange 	((void*)0x80001800, Bin_JumpCode_Length);
		memmove 			((void*)0x80001800, Bin_JumpCode, Bin_JumpCode_Length);
		DCFlushRangeNoSync 	((void*)0x80001800, Bin_JumpCode_Length);
		
		ICInvalidateRange 	((void*)0x90000020, Bin_JumpCode_Length);
		memmove 			((void*)0x90000020, Bin_JumpCode, Bin_JumpCode_Length);
		DCFlushRangeNoSync 	((void*)0x90000020, Bin_JumpCode_Length);
		
		WHL_Common->ep_Reboot = (void (*)()) LoadDOL (Dol_Reboot);
	}
	else if (strcmp (WHL_Common->Magic, WHL_MAGIC) == 0)
	{
		INIT_WHL_Common_Ressources(WHL_Common);
		WHL_Common->ep_Reboot		=  (void (*)()) 0x80001800;	
	}
	
	WHL_Common->Data = (u8*) Data;
	EntryPoint = (void (*)()) LoadDOL (Dol_WiiHbLoader);

	BeforeExit();
	SYS_ResetSystem(SYS_SHUTDOWN, 0, 0);
	_CPU_ISR_Disable (Level);
	EntryPoint();
	Reload();
	return ERROR_UNKNOW;
}

#endif

///////////////////////////////////////////////////////////////////////////////

#ifndef APP_MODE_LITE

void * C_App::LoadDOL (const void * Dol) 
{
	u32 i;
	T_DolHeader * dolfile ;

	if (!Dol) { return NULL; }
	dolfile = (T_DolHeader *) Dol;
	
	for (i = 0; i < 7; i++) 
	{
		if ((!dolfile->text_size[i]) || (dolfile->text_start[i] < 0x100)) continue;
		ICInvalidateRange ((void *) dolfile->text_start[i], dolfile->text_size[i]);
		memmove ((void *) dolfile->text_start[i], (u8*)Dol+dolfile->text_pos[i], dolfile->text_size[i]);
	}
	
	for(i = 0; i < 11; i++) 
	{
		if ((!dolfile->data_size[i]) || (dolfile->data_start[i] < 0x100)) continue;
		memmove ((void*) dolfile->data_start[i], (u8*)Dol+dolfile->data_pos[i], dolfile->data_size[i]);
		DCFlushRangeNoSync ((void *) dolfile->data_start[i], dolfile->data_size[i]);
	}
	
	memset ((void *) dolfile->bss_start, 0, dolfile->bss_size);
	DCFlushRange((void *) dolfile->bss_start, dolfile->bss_size);
    return (void*)dolfile->entry_point;
}

#endif

///////////////////////////////////////////////////////////////////////////////

